@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -36,7 +34,10 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-struct _EphyAboutHandlerPrivate {
+struct _EphyAboutHandler
+{
+  GObject parent_instance;
+
   EphySMaps *smaps;
 };
 
@@ -50,7 +51,7 @@ ephy_about_handler_finalize (GObject *object)
 {
   EphyAboutHandler *handler = EPHY_ABOUT_HANDLER (object);
 
-  g_clear_object (&handler->priv->smaps);
+  g_clear_object (&handler->smaps);
 
   G_OBJECT_CLASS (ephy_about_handler_parent_class)->finalize (object);
 }
@@ -58,7 +59,6 @@ ephy_about_handler_finalize (GObject *object)
 static void
 ephy_about_handler_init (EphyAboutHandler *handler)
 {
-  handler->priv = G_TYPE_INSTANCE_GET_PRIVATE (handler, EPHY_TYPE_ABOUT_HANDLER, EphyAboutHandlerPrivate);
 }
 
 static void
@@ -67,22 +67,21 @@ ephy_about_handler_class_init (EphyAboutHandlerClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->finalize = ephy_about_handler_finalize;
-  g_type_class_add_private (object_class, sizeof (EphyAboutHandlerPrivate));
 }
 
 static EphySMaps *
 ephy_about_handler_get_smaps (EphyAboutHandler *handler)
 {
-  if (!handler->priv->smaps)
-    handler->priv->smaps = ephy_smaps_new ();
+  if (!handler->smaps)
+    handler->smaps = ephy_smaps_new ();
 
-  return handler->priv->smaps;
+  return handler->smaps;
 }
 
 static void
 ephy_about_handler_finish_request (WebKitURISchemeRequest *request,
                                    gchar *data,
-                                   gssize data_length)
+                                   gsize data_length)
 {
   GInputStream *stream;
 
@@ -419,7 +418,7 @@ got_snapshot_path_for_url_cb (EphySnapshotService *service,
 
   snapshot = ephy_snapshot_service_get_snapshot_path_for_url_finish (service, result, NULL);
   if (snapshot) {
-    ephy_embed_shell_set_thumbanil_path (ephy_embed_shell_get_default (), data->url, data->mtime, snapshot);
+    ephy_embed_shell_set_thumbnail_path (ephy_embed_shell_get_default (), data->url, data->mtime, snapshot);
     g_free (snapshot);
   }
   g_free (data->url);
@@ -534,7 +533,6 @@ ephy_about_handler_handle_html_overview (EphyAboutHandler *handler,
   query->sort_type = EPHY_HISTORY_SORT_MOST_VISITED;
   query->limit = EPHY_ABOUT_OVERVIEW_MAX_ITEMS;
   query->ignore_hidden = TRUE;
-  query->ignore_local = TRUE;
   ephy_history_service_query_urls (history, query, NULL,
                                    (EphyHistoryJobCallback)history_service_query_urls_cb,
                                    g_object_ref (request));
