@@ -17,9 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Epiphany; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA  02110-1301  USA
+ * along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -37,27 +35,23 @@
 #include <glib/gi18n.h>
 #include <string.h>
 
-G_DEFINE_TYPE (EphyDownload, ephy_download, G_TYPE_OBJECT)
-
-#define EPHY_DOWNLOAD_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), EPHY_TYPE_DOWNLOAD, EphyDownloadPrivate))
-
-struct _EphyDownloadPrivate
+struct _EphyDownload
 {
+  GObject parent_instance;
+
   WebKitDownload *download;
 
   char *destination;
-  char *source;
+  char *content_type;
 
   EphyDownloadActionType action;
   guint32 start_time;
-
-  GtkWindow *window;
-  GtkWidget *widget;
-
-  guint inhibitor_cookie;
+  gboolean finished;
+  GError *error;
 };
 
+G_DEFINE_TYPE (EphyDownload, ephy_download, G_TYPE_OBJECT)
+
 enum
 {
   PROP_0,
@@ -65,29 +59,31 @@ enum
   PROP_DESTINATION,
   PROP_ACTION,
   PROP_START_TIME,
-  PROP_WINDOW,
-  PROP_WIDGET
+  PROP_CONTENT_TYPE,
+  LAST_PROP
 };
 
+static GParamSpec *obj_properties[LAST_PROP];
+
+enum
+{
+  FILENAME_SUGGESTED,
+  ERROR,
+  COMPLETED,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 static void
 ephy_download_get_property (GObject    *object,
                             guint       property_id,
                             GValue     *value,
                             GParamSpec *pspec)
 {
-  EphyDownload *download;
-  EphyDownloadPrivate *priv;
-
-  download = EPHY_DOWNLOAD (object);
-  priv = download->priv;
+  EphyDownload *download = EPHY_DOWNLOAD (object);
 
   switch (property_id) {
-    case PROP_WIDGET:
-      g_value_set_object (value, priv->widget);
-      break;
-    case PROP_WINDOW:
-      g_value_set_object (value, priv->window);
-      break;
     case PROP_DOWNLOAD:
       g_value_set_object (value, ephy_download_get_webkit_download (download));
       break;
@@ -100,6 +96,9 @@ ephy_download_get_property (GObject    *object,
     case PROP_START_TIME:
       g_value_set_uint (value, ephy_download_get_start_time (download));
       break;
+    case PROP_CONTENT_TYPE:
+      g_value_set_string (value, ephy_download_get_content_type (download));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -122,12 +121,6 @@ ephy_download_set_property (GObject      *object,
     case PROP_ACTION:
       ephy_download_set_action (download, g_value_get_enum (value));
       break;
-    case PROP_WINDOW:
-      download->priv->window = g_value_dup_object (value);
-      break;
-    case PROP_WIDGET:
-      ephy_download_set_widget (download, g_value_get_object (value));
-      break;
     case PROP_DOWNLOAD:
     case PROP_START_TIME:
     default:
@@ -143,51 +136,16 @@ ephy_download_set_property (GObject      *object,
  * Gets content-type information for @download. If the server didn't
  * provide a content type, the destination file is queried.
  *
- * Returns: content-type for @download, must be freed with g_free()
+ * Returns: content-type for @download
  **/
-char *
+const char *
 ephy_download_get_content_type (EphyDownload *download)
 {
-  WebKitURIResponse *response;
-  const char *destination_uri;
-  GFile *destination;
-  GFileInfo *info;
-  char *content_type = NULL;
-  GError *error = NULL;
+  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
 
-  response = webkit_download_get_response (download->priv->download);
-  if (response) {
-    content_type = g_strdup (webkit_uri_response_get_mime_type (response));
-
-    LOG ("ephy_download_get_content_type: WebKit: %s", content_type);
-
-    if (content_type)
-      return content_type;
-  }
-
-  destination_uri = webkit_download_get_destination (download->priv->download);
-  if (!destination_uri)
-    return NULL;
-
-  destination = g_file_new_for_uri (destination_uri);
-  info = g_file_query_info (destination, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-                            G_FILE_QUERY_INFO_NONE, NULL, &error);
-  if (info) {
-    content_type = g_strdup (g_file_info_get_content_type (info));
-    LOG ("ephy_download_get_content_type: GIO: %s", content_type);
-    g_object_unref (info);
-  } else {
-    LOG ("ephy_download_get_content_type: error getting file "
-         "content-type: %s", error->message);
-    g_error_free (error);
-  }
-
-  g_object_unref (destination);
-
-  return content_type;
+  return download->content_type;
 }
 
-
 /* Helper function to decide what EphyDownloadActionType should be the
  * default for the download. This implies that you want something to
  * happen, this function will never return EPHY_DOWNLOAD_ACTION_NONE.
@@ -330,7 +288,7 @@ set_destination_uri_for_suggested_filename (EphyDownload *download, const char *
   g_free (destination_filename);
 
   g_assert (destination_uri);
-  webkit_download_set_destination (download->priv->download, destination_uri);
+  webkit_download_set_destination (download->download, destination_uri);
   g_free (destination_uri);
 
   return TRUE;
@@ -351,8 +309,8 @@ ephy_download_set_destination_uri (EphyDownload *download,
   g_return_if_fail (EPHY_IS_DOWNLOAD (download));
   g_return_if_fail (destination != NULL);
 
-  webkit_download_set_destination (download->priv->download, destination);
-  g_object_notify (G_OBJECT (download), "destination");
+  webkit_download_set_destination (download->download, destination);
+  g_object_notify_by_pspec (G_OBJECT (download), obj_properties[PROP_DESTINATION]);
 }
 
 /**
@@ -370,48 +328,8 @@ ephy_download_set_action (EphyDownload *download,
 {
   g_return_if_fail (EPHY_IS_DOWNLOAD (download));
 
-  download->priv->action = action;
-  g_object_notify (G_OBJECT (download), "action");
-}
-
-/**
- * ephy_download_set_widget:
- * @download: an #EphyDownload
- * @widget: a #GtkWidget
- *
- * Sets @widget to be associated with @download as its UI.
- **/
-void
-ephy_download_set_widget (EphyDownload *download,
-                          GtkWidget *widget)
-{
-  g_return_if_fail (EPHY_IS_DOWNLOAD (download));
-
-  if (download->priv->widget != NULL)
-    g_object_unref (download->priv->widget);
-
-  download->priv->widget = NULL;
-
-  if (widget != NULL)
-    download->priv->widget = g_object_ref (widget);
-
-  g_object_notify (G_OBJECT (download), "widget");
-}
-
-/**
- * ephy_download_get_widget:
- * @download: an #EphyDownload
- *
- * Gets the #GtkWidget associated to this download.
- *
- * Returns: (transfer none): a #GtkWidget.
- **/
-GtkWidget *
-ephy_download_get_widget (EphyDownload *download)
-{
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
-
-  return download->priv->widget;
+  download->action = action;
+  g_object_notify_by_pspec (G_OBJECT (download), obj_properties[PROP_ACTION]);
 }
 
 /**
@@ -427,24 +345,7 @@ ephy_download_get_webkit_download (EphyDownload *download)
 {
   g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
 
-  return download->priv->download;
-}
-
-/**
- * ephy_download_get_window:
- * @download: an #EphyDownload
- *
- * Gets the window set as the parent of @download, this can be %NULL if no
- * specific window generated this download.
- *
- * Returns: (transfer none): a #GtkWindow
- **/
-GtkWindow *
-ephy_download_get_window (EphyDownload *download)
-{
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
-
-  return download->priv->window;
+  return download->download;
 }
 
 /**
@@ -460,7 +361,7 @@ ephy_download_get_destination_uri (EphyDownload *download)
 {
   g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
 
-  return webkit_download_get_destination (download->priv->download);
+  return webkit_download_get_destination (download->download);
 }
 
 /**
@@ -479,7 +380,7 @@ ephy_download_get_action (EphyDownload *download)
 {
   g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), EPHY_DOWNLOAD_ACTION_NONE);
 
-  return download->priv->action;
+  return download->action;
 }
 
 /**
@@ -496,41 +397,7 @@ ephy_download_get_start_time (EphyDownload *download)
 {
   g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), 0);
 
-  return download->priv->start_time;
-}
-
-static void
-acquire_session_inhibitor (EphyDownload *download)
-{
-  EphyDownloadPrivate *priv;
-  EphyEmbedShell *shell;
-
-  priv = download->priv;
-  shell = ephy_embed_shell_get_default ();
-
-  if (priv->inhibitor_cookie)
-    return;
-
-  priv->inhibitor_cookie = gtk_application_inhibit (GTK_APPLICATION (shell),
-                                                    priv->window,
-                                                    GTK_APPLICATION_INHIBIT_LOGOUT | GTK_APPLICATION_INHIBIT_SUSPEND,
-                                                    "Downloading");
-}
-
-static void
-release_session_inhibitor (EphyDownload *download)
-{
-  EphyDownloadPrivate *priv;
-  EphyEmbedShell *shell;
-
-  priv = download->priv;
-  shell = ephy_embed_shell_get_default ();
-
-  if (!priv->inhibitor_cookie)
-    return;
-
-  gtk_application_uninhibit (GTK_APPLICATION (shell), priv->inhibitor_cookie);
-  priv->inhibitor_cookie = 0;
+  return download->start_time;
 }
 
 /**
@@ -544,7 +411,38 @@ ephy_download_cancel (EphyDownload *download)
 {
   g_return_if_fail (EPHY_IS_DOWNLOAD (download));
 
-  webkit_download_cancel (download->priv->download);
+  webkit_download_cancel (download->download);
+}
+
+gboolean
+ephy_download_is_active (EphyDownload *download)
+{
+  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), FALSE);
+
+  return !download->finished;
+}
+
+gboolean
+ephy_download_succeeded (EphyDownload *download)
+{
+  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), FALSE);
+
+  return download->finished && !download->error;
+}
+
+gboolean
+ephy_download_failed (EphyDownload *download,
+                      GError      **error)
+{
+  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), FALSE);
+
+  if (download->finished && download->error) {
+    if (error)
+      *error = download->error;
+    return TRUE;
+  }
+
+  return FALSE;
 }
 
 /**
@@ -565,27 +463,24 @@ ephy_download_do_download_action (EphyDownload *download,
 {
     GFile *destination;
     const char *destination_uri;
-    EphyDownloadPrivate *priv;
     gboolean ret = FALSE;
 
-    priv = download->priv;
-
-    destination_uri = webkit_download_get_destination (priv->download);
+    destination_uri = webkit_download_get_destination (download->download);
     destination = g_file_new_for_uri (destination_uri);
 
-    switch ((action ? action : priv->action)) {
+    switch ((action ? action : download->action)) {
       case EPHY_DOWNLOAD_ACTION_AUTO:
         LOG ("ephy_download_do_download_action: auto");
         ret = ephy_download_do_download_action (download, decide_action_from_mime (download));
         break;
       case EPHY_DOWNLOAD_ACTION_BROWSE_TO:
         LOG ("ephy_download_do_download_action: browse_to");
-        ret = ephy_file_browse_to (destination, priv->start_time);
+        ret = ephy_file_browse_to (destination, download->start_time);
         break;
       case EPHY_DOWNLOAD_ACTION_OPEN:
         LOG ("ephy_download_do_download_action: open");
         ret = ephy_embed_shell_launch_handler (ephy_embed_shell_get_default (), 
-                                               destination, NULL, priv->start_time);
+                                               destination, NULL, download->start_time);
         break;
       case EPHY_DOWNLOAD_ACTION_NONE:
         LOG ("ephy_download_do_download_action: none");
@@ -609,29 +504,17 @@ static void
 ephy_download_dispose (GObject *object)
 {
   EphyDownload *download = EPHY_DOWNLOAD (object);
-  EphyDownloadPrivate *priv;
 
   LOG ("EphyDownload disposed %p", object);
 
-  priv = download->priv;
-
-  release_session_inhibitor (download);
-
-  if (priv->download) {
-    g_signal_handlers_disconnect_matched (priv->download, G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, download);
-    g_object_unref (priv->download);
-    priv->download = NULL;
+  if (download->download) {
+    g_signal_handlers_disconnect_matched (download->download, G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, download);
+    g_object_unref (download->download);
+    download->download = NULL;
   }
 
-  if (priv->window) {
-    g_object_unref (priv->window);
-    priv->window = NULL;
-  }
-
-  if (priv->widget) {
-    g_object_unref (priv->widget);
-    priv->widget = NULL;
-  }
+  g_clear_error(&download->error);
+  g_clear_pointer (&download->content_type, g_free);
 
   G_OBJECT_CLASS (ephy_download_parent_class)->dispose (object);
 }
@@ -641,8 +524,6 @@ ephy_download_class_init (EphyDownloadClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (EphyDownloadPrivate));
-
   object_class->get_property = ephy_download_get_property;
   object_class->set_property = ephy_download_set_property;
   object_class->dispose = ephy_download_dispose;
@@ -652,30 +533,26 @@ ephy_download_class_init (EphyDownloadClass *klass)
    *
    * Internal WebKitDownload.
    */
-  g_object_class_install_property (object_class, PROP_DOWNLOAD,
-                                   g_param_spec_object ("download",
-                                                        "Internal WebKitDownload",
-                                                        "The WebKitDownload used internally by EphyDownload",
-                                                        WEBKIT_TYPE_DOWNLOAD,
-                                                        G_PARAM_READABLE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_DOWNLOAD] =
+    g_param_spec_object ("download",
+                        "Internal WebKitDownload",
+                        "The WebKitDownload used internally by EphyDownload",
+                        WEBKIT_TYPE_DOWNLOAD,
+                        G_PARAM_READABLE |
+                        G_PARAM_STATIC_STRINGS);
 
   /**
    * EphyDownload::destination:
    *
    * The destination URI where to store the download.
    */
-  g_object_class_install_property (object_class, PROP_DESTINATION,
-                                   g_param_spec_string ("destination",
-                                                        "Destination",
-                                                        "Destination file URI",
-                                                        NULL,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_DESTINATION] =
+    g_param_spec_string ("destination",
+                        "Destination",
+                        "Destination file URI",
+                        NULL,
+                        G_PARAM_READWRITE |
+                        G_PARAM_STATIC_STRINGS);
 
   /**
    * EphyDownload::action:
@@ -684,16 +561,14 @@ ephy_download_class_init (EphyDownloadClass *klass)
    * open files" is enabled, or when ephy_download_do_download_action () is
    * called.
    */
-  g_object_class_install_property (object_class, PROP_ACTION,
-                                   g_param_spec_enum ("action",
-                                                      "Download action",
-                                                      "Action to take when download finishes",
-                                                      EPHY_TYPE_DOWNLOAD_ACTION_TYPE,
-                                                      EPHY_DOWNLOAD_ACTION_NONE,
-                                                      G_PARAM_READABLE |
-                                                      G_PARAM_STATIC_NAME |
-                                                      G_PARAM_STATIC_NICK |
-                                                      G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_ACTION] =
+    g_param_spec_enum ("action",
+                      "Download action",
+                      "Action to take when download finishes",
+                      EPHY_TYPE_DOWNLOAD_ACTION_TYPE,
+                      EPHY_DOWNLOAD_ACTION_NONE,
+                      G_PARAM_READABLE |
+                      G_PARAM_STATIC_STRINGS);
 
   /**
    * EphyDownload::start-time:
@@ -701,61 +576,35 @@ ephy_download_class_init (EphyDownloadClass *klass)
    * User time when the download started, useful for launching applications
    * aware of focus stealing.
    */
-  g_object_class_install_property (object_class, PROP_START_TIME,
-                                   g_param_spec_uint ("start-time",
-                                                      "Event start time",
-                                                      "Time for focus-stealing prevention.",
-                                                      0, G_MAXUINT32, 0,
-                                                      G_PARAM_READABLE |
-                                                      G_PARAM_STATIC_NAME |
-                                                      G_PARAM_STATIC_NICK |
-                                                      G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_START_TIME] =
+    g_param_spec_uint ("start-time",
+                      "Event start time",
+                      "Time for focus-stealing prevention.",
+                      0, G_MAXUINT32, 0,
+                      G_PARAM_READABLE |
+                      G_PARAM_STATIC_STRINGS);
+
+  obj_properties[PROP_CONTENT_TYPE] =
+    g_param_spec_string ("content-type",
+                        "Content Type",
+                        "The download content type",
+                        NULL,
+                        G_PARAM_READABLE |
+                        G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 
   /**
-   * EphyDownload:window:
-   *
-   * Window that produced the download, the download will be shown in its
-   * parent window.
-   */
-  g_object_class_install_property (object_class, PROP_WINDOW,
-                                   g_param_spec_object ("window",
-                                                        "A GtkWindow",
-                                                        "Window that produced this download.",
-                                                        GTK_TYPE_WINDOW,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
-
-  /**
-   * EphyDownload::widget:
-   *
-   * An EphyDownloadWidget -or any other GtkWidget- that is representing this
-   * EphyDownload to the user.
-   */
-  g_object_class_install_property (object_class, PROP_WIDGET,
-                                   g_param_spec_object ("widget",
-                                                        "A GtkWidget",
-                                                        "GtkWidget showing this download.",
-                                                        GTK_TYPE_WIDGET,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
-
-  /**
-   * EphyDownload::completed:
+   * EphyDownload::filename-suggested:
    *
    * The ::filename-suggested signal is emitted when we have received the
    * suggested filename from WebKit.
    **/
-  g_signal_new ("filename-suggested",
+  signals[FILENAME_SUGGESTED] = g_signal_new ("filename-suggested",
                 G_OBJECT_CLASS_TYPE (object_class),
                 G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (EphyDownloadClass, filename_suggested),
-                NULL, NULL,
-                g_cclosure_marshal_generic,
+                0,
+                NULL, NULL, NULL,
                 G_TYPE_NONE,
                 1,
                 G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
@@ -765,12 +614,11 @@ ephy_download_class_init (EphyDownloadClass *klass)
    *
    * The ::completed signal is emitted when @download has finished downloading.
    **/
-  g_signal_new ("completed",
+  signals[COMPLETED] = g_signal_new ("completed",
                 G_OBJECT_CLASS_TYPE (object_class),
                 G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (EphyDownloadClass, completed),
-                NULL, NULL,
-                g_cclosure_marshal_VOID__VOID,
+                0,
+                NULL, NULL, NULL,
                 G_TYPE_NONE,
                 0);
   /**
@@ -778,39 +626,43 @@ ephy_download_class_init (EphyDownloadClass *klass)
    *
    * The ::error signal wraps the @download ::error signal.
    **/
-  g_signal_new ("error",
+  signals[ERROR] = g_signal_new ("error",
                 G_OBJECT_CLASS_TYPE (object_class),
                 G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET (EphyDownloadClass, error),
-                NULL, NULL,
-                g_cclosure_marshal_generic,
+                0,
+                NULL, NULL, NULL,
                 G_TYPE_NONE,
-                0);
+                1, G_TYPE_POINTER);
 }
 
 static void
 ephy_download_init (EphyDownload *download)
 {
-  download->priv = EPHY_DOWNLOAD_GET_PRIVATE (download);
-
   LOG ("EphyDownload initialising %p", download);
 
-  download->priv->download = NULL;
+  download->download = NULL;
 
-  download->priv->action = EPHY_DOWNLOAD_ACTION_NONE;
+  download->action = EPHY_DOWNLOAD_ACTION_NONE;
 
-  download->priv->start_time = gtk_get_current_event_time ();
-
-  download->priv->window = NULL;
-  download->priv->widget = NULL;
+  download->start_time = gtk_get_current_event_time ();
 }
 
 static void
-download_created_destination_cb (WebKitDownload *wk_download,
-                                 const gchar *destination,
-                                 EphyDownload *download)
+download_response_changed_cb (WebKitDownload *wk_download,
+                              GParamSpec *spec,
+                              EphyDownload *download)
 {
-  acquire_session_inhibitor (download);
+  WebKitURIResponse *response;
+  const char *mime_type;
+
+  response = webkit_download_get_response (download->download);
+  mime_type = webkit_uri_response_get_mime_type (response);
+  if (!mime_type)
+    return;
+
+  download->content_type = g_content_type_from_mime_type (mime_type);
+  if (download->content_type)
+    g_object_notify_by_pspec (G_OBJECT (download), obj_properties[PROP_CONTENT_TYPE]);
 }
 
 static gboolean
@@ -821,7 +673,7 @@ download_decide_destination_cb (WebKitDownload *wk_download,
   if (webkit_download_get_destination (wk_download))
     return TRUE;
 
-  g_signal_emit_by_name (download, "filename-suggested", suggested_filename);
+  g_signal_emit (download, signals[FILENAME_SUGGESTED], 0, suggested_filename);
 
   if (webkit_download_get_destination (wk_download))
     return TRUE;
@@ -829,23 +681,62 @@ download_decide_destination_cb (WebKitDownload *wk_download,
   return set_destination_uri_for_suggested_filename (download, suggested_filename);
 }
 
+static void
+download_created_destination_cb (WebKitDownload *wk_download,
+                                 const char *destination,
+                                 EphyDownload *download)
+{
+  char *filename;
+  char *content_type;
+
+  if (download->content_type && !g_content_type_is_unknown (download->content_type))
+    return;
+
+  /* The server didn't provide a valid content type, let's try to guess it from the
+   * destination filename. We use g_content_type_guess() here instead of g_file_query_info(),
+   * because we are only using the filename to guess the content type, since it doesn't make
+   * sense to sniff the destination URI that will be empty until the download is completed.
+   * We can't use g_file_query_info() with the partial download file either, because it will
+   * always return application/x-partial-download based on the .wkdownload extension.
+   */
+  filename = g_filename_from_uri (destination, NULL, NULL);
+  if (!filename)
+    return;
+
+  content_type = g_content_type_guess (filename, NULL, 0, NULL);
+  g_free (filename);
+
+  if (g_content_type_is_unknown (content_type)) {
+    /* We could try to connect to received-data signal and sniff the contents when we have
+     * enough data written in the file, but I don't think it's worth it.
+     */
+    g_free (content_type);
+    return;
+  }
+
+  if (!download->content_type ||
+      (download->content_type && !g_content_type_equals (download->content_type, content_type))) {
+    g_free (download->content_type);
+    download->content_type = content_type;
+    g_object_notify_by_pspec (G_OBJECT (download), obj_properties[PROP_CONTENT_TYPE]);
+    return;
+  }
+
+  g_free (content_type);
+}
+
 static void
 download_finished_cb (WebKitDownload *wk_download,
                       EphyDownload *download)
 {
-  EphyDownloadPrivate *priv;
-
-  priv = download->priv;
-
-  g_signal_emit_by_name (download, "completed");
+  download->finished = TRUE;
+  g_signal_emit (download, signals[COMPLETED], 0);
 
   if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_AUTO_DOWNLOADS) &&
-      priv->action == EPHY_DOWNLOAD_ACTION_NONE)
+      download->action == EPHY_DOWNLOAD_ACTION_NONE)
     ephy_download_do_download_action (download, EPHY_DOWNLOAD_ACTION_AUTO);
   else
-    ephy_download_do_download_action (download, priv->action);
-
-  release_session_inhibitor (download);
+    ephy_download_do_download_action (download, download->action);
 }
 
 static void
@@ -853,41 +744,40 @@ download_failed_cb (WebKitDownload *wk_download,
                     GError *error,
                     EphyDownload *download)
 {
-  gboolean ret = FALSE;
-
   g_signal_handlers_disconnect_by_func (wk_download, download_finished_cb, download);
 
   LOG ("error (%d - %d)! %s", error->code, 0, error->message);
-  g_signal_emit_by_name (download, "error", 0, error->code, error->message, &ret);
-
-  release_session_inhibitor (download);
+  download->finished = TRUE;
+  download->error = g_error_copy (error);
+  g_signal_emit (download, signals[ERROR], 0, download->error);
 }
 
 /**
  * ephy_download_new:
  * @download: a #WebKitDownload to wrap
- * @parent: the #GtkWindow parent of the download, or %NULL
  *
  * Wraps @download in an #EphyDownload.
  *
  * Returns: an #EphyDownload.
  **/
 EphyDownload *
-ephy_download_new (WebKitDownload *download,
-                   GtkWindow *parent)
+ephy_download_new (WebKitDownload *download)
 {
   EphyDownload *ephy_download;
 
   g_return_val_if_fail (WEBKIT_IS_DOWNLOAD (download), NULL);
 
-  ephy_download = g_object_new (EPHY_TYPE_DOWNLOAD, "window", parent, NULL);
+  ephy_download = g_object_new (EPHY_TYPE_DOWNLOAD, NULL);
 
-  g_signal_connect (download, "created-destination",
-                    G_CALLBACK (download_created_destination_cb),
+  g_signal_connect (download, "notify::response",
+                    G_CALLBACK (download_response_changed_cb),
                     ephy_download);
   g_signal_connect (download, "decide-destination",
                     G_CALLBACK (download_decide_destination_cb),
                     ephy_download);
+  g_signal_connect (download, "created-destination",
+                    G_CALLBACK (download_created_destination_cb),
+                    ephy_download);
   g_signal_connect (download, "finished",
                     G_CALLBACK (download_finished_cb),
                     ephy_download);
@@ -895,7 +785,7 @@ ephy_download_new (WebKitDownload *download,
                     G_CALLBACK (download_failed_cb),
                     ephy_download);
 
-  ephy_download->priv->download = g_object_ref (download);
+  ephy_download->download = g_object_ref (download);
   g_object_set_data (G_OBJECT (download), "ephy-download-set", GINT_TO_POINTER (TRUE));
 
   return ephy_download;
@@ -904,15 +794,13 @@ ephy_download_new (WebKitDownload *download,
 /**
  * ephy_download_new_for_uri:
  * @uri: a source URI from where to download
- * @parent: the #GtkWindow parent of the download, or %NULL
  *
  * Creates an #EphyDownload to download @uri.
  *
  * Returns: an #EphyDownload.
  **/
 EphyDownload *
-ephy_download_new_for_uri (const char *uri,
-                           GtkWindow *parent)
+ephy_download_new_for_uri (const char *uri)
 {
   EphyDownload *ephy_download;
   WebKitDownload *download;
@@ -921,8 +809,7 @@ ephy_download_new_for_uri (const char *uri,
   g_return_val_if_fail (uri != NULL, NULL);
 
   download = webkit_web_context_download_uri (ephy_embed_shell_get_web_context (shell), uri);
-
-  ephy_download = ephy_download_new (download, parent);
+  ephy_download = ephy_download_new (download);
   g_object_unref (download);
 
   return ephy_download;
