@@ -12,9 +12,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
@@ -32,17 +30,11 @@
 
 G_BEGIN_DECLS
 
-#define EPHY_TYPE_EMBED_CONTAINER               (ephy_embed_container_get_type ())
-#define EPHY_EMBED_CONTAINER(o)                 (G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_EMBED_CONTAINER, EphyEmbedContainer))
-#define EPHY_EMBED_CONTAINER_IFACE(k)           (G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_EMBED_CONTAINER, EphyEmbedContainerIface))
-#define EPHY_IS_EMBED_CONTAINER(o)              (G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_EMBED_CONTAINER))
-#define EPHY_IS_EMBED_CONTAINER_IFACE(k)        (G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED_CONTAINER))
-#define EPHY_EMBED_CONTAINER_GET_IFACE(inst)    (G_TYPE_INSTANCE_GET_INTERFACE ((inst), EPHY_TYPE_EMBED_CONTAINER, EphyEmbedContainerIface))
+#define EPHY_TYPE_EMBED_CONTAINER (ephy_embed_container_get_type ())
 
-typedef struct _EphyEmbedContainer      EphyEmbedContainer;
-typedef struct _EphyEmbedContainerIface EphyEmbedContainerIface;
+G_DECLARE_INTERFACE (EphyEmbedContainer, ephy_embed_container, EPHY, EMBED_CONTAINER, GObject)
 
-struct _EphyEmbedContainerIface
+struct _EphyEmbedContainerInterface
 {
   GTypeInterface parent_iface;
 
@@ -64,7 +56,6 @@ struct _EphyEmbedContainerIface
   gboolean (* get_is_popup)        (EphyEmbedContainer *container);
 };
 
-GType             ephy_embed_container_get_type         (void);
 gint              ephy_embed_container_add_child        (EphyEmbedContainer *container,
                                                          EphyEmbed          *child,
                                                          gint                position,
