@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __EPHY_EMBED_PREFS_H__
@@ -34,7 +32,7 @@
 
 G_BEGIN_DECLS
 
-WebKitWebViewGroup *ephy_embed_prefs_get_web_view_group (void);
+WebKitSettings *ephy_embed_prefs_get_settings (void);
 void ephy_embed_prefs_set_cookie_accept_policy          (WebKitCookieManager *cookie_manager,
                                                          const char          *settings_policy);
 
