@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -46,10 +44,7 @@
 #define PRINT_SETTINGS_FILENAME "print-settings.ini"
 #define OVERVIEW_RELOAD_DELAY 500
 
-#define EPHY_EMBED_SHELL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellPrivate))
-
-struct _EphyEmbedShellPrivate
-{
+typedef struct {
   WebKitWebContext *web_context;
   EphyHistoryService *global_history_service;
   EphyEncodings *encodings;
@@ -57,13 +52,14 @@ struct _EphyEmbedShellPrivate
   GtkPrintSettings *print_settings;
   EphyEmbedShellMode mode;
   WebKitUserContentManager *user_content;
+  EphyDownloadsManager *downloads_manager;
   EphyAboutHandler *about_handler;
   guint update_overview_timeout_id;
   guint hiding_overview_item;
   GDBusConnection *bus;
   GList *web_extensions;
   guint web_extensions_page_created_signal_id;
-};
+} EphyEmbedShellPrivate;
 
 enum
 {
@@ -88,14 +84,14 @@ enum
 
 static GParamSpec *object_properties[N_PROPERTIES] = { NULL, };
 
-EphyEmbedShell *embed_shell = NULL;
+static EphyEmbedShell *embed_shell = NULL;
 
-G_DEFINE_TYPE (EphyEmbedShell, ephy_embed_shell, GTK_TYPE_APPLICATION)
+G_DEFINE_TYPE_WITH_PRIVATE (EphyEmbedShell, ephy_embed_shell, GTK_TYPE_APPLICATION)
 
 static void
 ephy_embed_shell_dispose (GObject *object)
 {
-  EphyEmbedShellPrivate *priv = EPHY_EMBED_SHELL (object)->priv;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
 
   if (priv->update_overview_timeout_id > 0) {
     g_source_remove (priv->update_overview_timeout_id);
@@ -108,6 +104,7 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->global_history_service);
   g_clear_object (&priv->about_handler);
   g_clear_object (&priv->user_content);
+  g_clear_object (&priv->downloads_manager);
   g_clear_object (&priv->web_context);
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
@@ -124,9 +121,10 @@ static EphyWebExtensionProxy *
 ephy_embed_shell_find_web_extension (EphyEmbedShell *shell,
                                      const char *name_owner)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GList *l;
 
-  l = g_list_find_custom (shell->priv->web_extensions, name_owner, (GCompareFunc)web_extension_compare);
+  l = g_list_find_custom (priv->web_extensions, name_owner, (GCompareFunc)web_extension_compare);
 
   if (!l)
     g_warning ("Could not find extension with name owner `%s´.", name_owner);
@@ -182,12 +180,13 @@ history_service_query_urls_cb (EphyHistoryService *service,
                                GList *urls,
                                EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GList *l;
 
   if (!success)
     return;
 
-  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+  for (l = priv->web_extensions; l; l = g_list_next (l)) {
     EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
 
     ephy_web_extension_proxy_history_set_urls (web_extension, urls);
@@ -197,15 +196,15 @@ history_service_query_urls_cb (EphyHistoryService *service,
 static void
 ephy_embed_shell_update_overview_urls (EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   EphyHistoryQuery *query;
 
   query = ephy_history_query_new ();
   query->sort_type = EPHY_HISTORY_SORT_MOST_VISITED;
   query->limit = EPHY_ABOUT_OVERVIEW_MAX_ITEMS;
   query->ignore_hidden = TRUE;
-  query->ignore_local = TRUE;
 
-  ephy_history_service_query_urls (shell->priv->global_history_service, query, NULL,
+  ephy_history_service_query_urls (priv->global_history_service, query, NULL,
                                    (EphyHistoryJobCallback) history_service_query_urls_cb,
                                    shell);
   ephy_history_query_free (query);
@@ -221,9 +220,11 @@ history_service_urls_visited_cb (EphyHistoryService *history,
 static gboolean
 ephy_embed_shell_update_overview_timeout_cb (EphyEmbedShell *shell)
 {
-  shell->priv->update_overview_timeout_id = 0;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  if (shell->priv->hiding_overview_item > 0)
+  priv->update_overview_timeout_id = 0;
+
+  if (priv->hiding_overview_item > 0)
     return FALSE;
 
   ephy_embed_shell_update_overview_urls (shell);
@@ -237,12 +238,14 @@ history_set_url_hidden_cb (EphyHistoryService *service,
                            gpointer result_data,
                            EphyEmbedShell *shell)
 {
-  shell->priv->hiding_overview_item--;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  priv->hiding_overview_item--;
 
   if (!success)
     return;
 
-  if (shell->priv->update_overview_timeout_id > 0)
+  if (priv->update_overview_timeout_id > 0)
     return;
 
   ephy_embed_shell_update_overview_urls (shell);
@@ -253,22 +256,23 @@ web_extension_overview_message_received_cb (WebKitUserContentManager *manager,
                                             WebKitJavascriptResult *message,
                                             EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   char *url_to_remove;
 
   url_to_remove = ephy_embed_utils_get_js_result_as_string (message);
 
-  shell->priv->hiding_overview_item++;
-  ephy_history_service_set_url_hidden (shell->priv->global_history_service,
+  priv->hiding_overview_item++;
+  ephy_history_service_set_url_hidden (priv->global_history_service,
                                        url_to_remove, TRUE, NULL,
                                        (EphyHistoryJobCallback) history_set_url_hidden_cb,
                                        shell);
   g_free (url_to_remove);
 
-  if (shell->priv->update_overview_timeout_id > 0)
-    g_source_remove (shell->priv->update_overview_timeout_id);
+  if (priv->update_overview_timeout_id > 0)
+    g_source_remove (priv->update_overview_timeout_id);
 
   /* Wait for the CSS animations to finish before refreshing */
-  shell->priv->update_overview_timeout_id =
+  priv->update_overview_timeout_id =
     g_timeout_add (OVERVIEW_RELOAD_DELAY, (GSourceFunc) ephy_embed_shell_update_overview_timeout_cb, shell);
 }
 
@@ -299,22 +303,25 @@ static void
 web_extension_destroyed (EphyEmbedShell *shell,
                          GObject *web_extension)
 {
-  shell->priv->web_extensions = g_list_remove (shell->priv->web_extensions, web_extension);
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  priv->web_extensions = g_list_remove (priv->web_extensions, web_extension);
 }
 
 static void
 ephy_embed_shell_watch_web_extension (EphyEmbedShell *shell,
                                       const char *web_extension_id)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   EphyWebExtensionProxy *web_extension;
   char *service_name;
 
-  if (!shell->priv->bus)
+  if (!priv->bus)
     return;
 
   service_name = g_strdup_printf ("%s-%s", EPHY_WEB_EXTENSION_SERVICE_NAME, web_extension_id);
-  web_extension = ephy_web_extension_proxy_new (shell->priv->bus, service_name);
-  shell->priv->web_extensions = g_list_prepend (shell->priv->web_extensions, web_extension);
+  web_extension = ephy_web_extension_proxy_new (priv->bus, service_name);
+  priv->web_extensions = g_list_prepend (priv->web_extensions, web_extension);
   g_object_weak_ref (G_OBJECT (web_extension), (GWeakNotify)web_extension_destroyed, shell);
   g_free (service_name);
 }
@@ -332,9 +339,10 @@ history_service_url_title_changed_cb (EphyHistoryService *service,
                                       const char *title,
                                       EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GList *l;
 
-  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+  for (l = priv->web_extensions; l; l = g_list_next (l)) {
     EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
 
     ephy_web_extension_proxy_history_set_url_title (web_extension, url, title);
@@ -346,9 +354,10 @@ history_service_url_deleted_cb (EphyHistoryService *service,
                                 const char *url,
                                 EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GList *l;
 
-  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+  for (l = priv->web_extensions; l; l = g_list_next (l)) {
     EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
 
     ephy_web_extension_proxy_history_delete_url (web_extension, url);
@@ -360,12 +369,13 @@ history_service_host_deleted_cb (EphyHistoryService *service,
                                  const char *deleted_url,
                                  EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GList *l;
   SoupURI *deleted_uri;
 
   deleted_uri = soup_uri_new (deleted_url);
 
-  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+  for (l = priv->web_extensions; l; l = g_list_next (l)) {
     EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
 
     ephy_web_extension_proxy_history_delete_host (web_extension, soup_uri_get_host (deleted_uri));
@@ -378,9 +388,10 @@ static void
 history_service_cleared_cb (EphyHistoryService *service,
                             EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GList *l;
 
-  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+  for (l = priv->web_extensions; l; l = g_list_next (l)) {
     EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
 
     ephy_web_extension_proxy_history_clear (web_extension);
@@ -388,14 +399,19 @@ history_service_cleared_cb (EphyHistoryService *service,
 }
 
 void
-ephy_embed_shell_set_thumbanil_path (EphyEmbedShell *shell,
+ephy_embed_shell_set_thumbnail_path (EphyEmbedShell *shell,
                                      const char *url,
                                      time_t mtime,
                                      const char *path)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GList *l;
 
-  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+  ephy_history_service_set_url_thumbnail_time (priv->global_history_service,
+                                               url, mtime,
+                                               NULL, NULL, NULL);
+
+  for (l = priv->web_extensions; l; l = g_list_next (l)) {
     EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
 
     ephy_web_extension_proxy_history_set_url_thumbnail (web_extension, url, path);
@@ -411,51 +427,36 @@ ephy_embed_shell_set_thumbanil_path (EphyEmbedShell *shell,
 GObject *
 ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
   g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
 
-  if (shell->priv->global_history_service == NULL) {
+  if (priv->global_history_service == NULL) {
     char *filename;
-    EphySQLiteConnectionMode mode;
-
-    if (shell->priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO ||
-        shell->priv->mode == EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER)
-      mode = EPHY_SQLITE_CONNECTION_MODE_READ_ONLY;
-    else
-      mode = EPHY_SQLITE_CONNECTION_MODE_READWRITE;
 
     filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
-    shell->priv->global_history_service = ephy_history_service_new (filename, mode);
+    priv->global_history_service = ephy_history_service_new (filename,
+                                                             priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO);
     g_free (filename);
-    g_return_val_if_fail (shell->priv->global_history_service, NULL);
-    g_signal_connect (shell->priv->global_history_service, "urls-visited",
+    g_return_val_if_fail (priv->global_history_service, NULL);
+    g_signal_connect (priv->global_history_service, "urls-visited",
                       G_CALLBACK (history_service_urls_visited_cb),
                       shell);
-    g_signal_connect (shell->priv->global_history_service, "url-title-changed",
+    g_signal_connect (priv->global_history_service, "url-title-changed",
                       G_CALLBACK (history_service_url_title_changed_cb),
                       shell);
-    g_signal_connect (shell->priv->global_history_service, "url-deleted",
+    g_signal_connect (priv->global_history_service, "url-deleted",
                       G_CALLBACK (history_service_url_deleted_cb),
                       shell);
-    g_signal_connect (shell->priv->global_history_service, "host-deleted",
+    g_signal_connect (priv->global_history_service, "host-deleted",
                       G_CALLBACK (history_service_host_deleted_cb),
                       shell);
-    g_signal_connect (shell->priv->global_history_service, "cleared",
+    g_signal_connect (priv->global_history_service, "cleared",
                       G_CALLBACK (history_service_cleared_cb),
                       shell);
   }
 
-  return G_OBJECT (shell->priv->global_history_service);
-}
-
-static void
-snapshot_saved_cb (EphySnapshotService *service,
-                   const char          *url,
-                   gint64               mtime,
-                   EphyEmbedShell      *shell)
-{
-  ephy_history_service_set_url_thumbnail_time (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (shell)),
-                                               url, mtime,
-                                               NULL, NULL, NULL);
+  return G_OBJECT (priv->global_history_service);
 }
 
 /**
@@ -467,12 +468,14 @@ snapshot_saved_cb (EphySnapshotService *service,
 GObject *
 ephy_embed_shell_get_encodings (EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
   g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
 
-  if (shell->priv->encodings == NULL)
-    shell->priv->encodings = ephy_encodings_new ();
+  if (priv->encodings == NULL)
+    priv->encodings = ephy_encodings_new ();
 
-  return G_OBJECT (shell->priv->encodings);
+  return G_OBJECT (priv->encodings);
 }
 
 void
@@ -491,7 +494,9 @@ static void
 about_request_cb (WebKitURISchemeRequest *request,
                   EphyEmbedShell *shell)
 {
-  ephy_about_handler_handle_request (shell->priv->about_handler, request);
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  ephy_about_handler_handle_request (priv->about_handler, request);
 }
 
 static void
@@ -523,6 +528,7 @@ static void
 initialize_web_extensions (WebKitWebContext* web_context,
                            EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   GVariant *user_data;
   gboolean private_profile;
   char *web_extension_id;
@@ -533,7 +539,7 @@ initialize_web_extensions (WebKitWebContext* web_context,
   web_extension_id = g_strdup_printf ("%u-%u", getpid (), ++web_extension_count);
   ephy_embed_shell_watch_web_extension (shell, web_extension_id);
 
-  private_profile = EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (shell->priv->mode);
+  private_profile = EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode);
   user_data = g_variant_new ("(ssb)", web_extension_id, ephy_dot_dir (), private_profile);
   webkit_web_context_set_web_extensions_initialization_user_data (web_context, user_data);
 }
@@ -541,14 +547,16 @@ initialize_web_extensions (WebKitWebContext* web_context,
 static void
 ephy_embed_shell_setup_web_extensions_connection (EphyEmbedShell *shell)
 {
-  shell->priv->bus = g_application_get_dbus_connection (G_APPLICATION (shell));
-  if (!shell->priv->bus) {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  priv->bus = g_application_get_dbus_connection (G_APPLICATION (shell));
+  if (!priv->bus) {
     g_warning ("Application not connected to session bus");
     return;
   }
 
-  shell->priv->web_extensions_page_created_signal_id =
-    g_dbus_connection_signal_subscribe (shell->priv->bus,
+  priv->web_extensions_page_created_signal_id =
+    g_dbus_connection_signal_subscribe (priv->bus,
                                         NULL,
                                         EPHY_WEB_EXTENSION_INTERFACE,
                                         "PageCreated",
@@ -563,6 +571,7 @@ ephy_embed_shell_setup_web_extensions_connection (EphyEmbedShell *shell)
 static void
 ephy_embed_shell_setup_process_model (EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   EphyPrefsProcessModel process_model;
   guint max_processes;
 
@@ -580,17 +589,17 @@ ephy_embed_shell_setup_process_model (EphyEmbedShell *shell)
     break;
   }
 
-  webkit_web_context_set_process_model (shell->priv->web_context, WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
-  webkit_web_context_set_web_process_count_limit (shell->priv->web_context, max_processes);
+  webkit_web_context_set_process_model (priv->web_context, WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
+  webkit_web_context_set_web_process_count_limit (priv->web_context, max_processes);
 }
 
 static void
 ephy_embed_shell_create_web_context (EphyEmbedShell *embed_shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (embed_shell);
   WebKitWebsiteDataManager *manager;
   char *data_dir;
   char *cache_dir;
-  EphyEmbedShellPrivate *priv = embed_shell->priv;
 
   data_dir = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode) ?
                                ephy_dot_dir () : g_get_user_data_dir (),
@@ -613,7 +622,7 @@ static void
 ephy_embed_shell_startup (GApplication* application)
 {
   EphyEmbedShell *shell = EPHY_EMBED_SHELL (application);
-  EphyEmbedShellPrivate *priv = shell->priv;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   char *favicon_db_path;
   WebKitCookieManager *cookie_manager;
   char *filename;
@@ -630,29 +639,29 @@ ephy_embed_shell_startup (GApplication* application)
 
   /* User content manager */
   if (priv->mode != EPHY_EMBED_SHELL_MODE_TEST)
-    shell->priv->user_content = webkit_user_content_manager_new ();
+    priv->user_content = webkit_user_content_manager_new ();
 
-  webkit_user_content_manager_register_script_message_handler (shell->priv->user_content,
+  webkit_user_content_manager_register_script_message_handler (priv->user_content,
                                                                "overview");
-  g_signal_connect (shell->priv->user_content, "script-message-received::overview",
+  g_signal_connect (priv->user_content, "script-message-received::overview",
                     G_CALLBACK (web_extension_overview_message_received_cb),
                     shell);
 
-  webkit_user_content_manager_register_script_message_handler (shell->priv->user_content,
+  webkit_user_content_manager_register_script_message_handler (priv->user_content,
                                                                "tlsErrorPage");
-  g_signal_connect (shell->priv->user_content, "script-message-received::tlsErrorPage",
+  g_signal_connect (priv->user_content, "script-message-received::tlsErrorPage",
                     G_CALLBACK (web_extension_tls_error_page_message_received_cb),
                     shell);
 
-  webkit_user_content_manager_register_script_message_handler (shell->priv->user_content,
+  webkit_user_content_manager_register_script_message_handler (priv->user_content,
                                                                "formAuthData");
-  g_signal_connect (shell->priv->user_content, "script-message-received::formAuthData",
+  g_signal_connect (priv->user_content, "script-message-received::formAuthData",
                     G_CALLBACK (web_extension_form_auth_data_message_received_cb),
                     shell);
 
-  webkit_user_content_manager_register_script_message_handler (shell->priv->user_content,
+  webkit_user_content_manager_register_script_message_handler (priv->user_content,
                                                                "aboutApps");
-  g_signal_connect (shell->priv->user_content, "script-message-received::aboutApps",
+  g_signal_connect (priv->user_content, "script-message-received::aboutApps",
                     G_CALLBACK (web_extension_about_apps_message_received_cb),
                     shell);
 
@@ -673,7 +682,7 @@ ephy_embed_shell_startup (GApplication* application)
 
 
   /* about: URIs handler */
-  shell->priv->about_handler = ephy_about_handler_new ();
+  priv->about_handler = ephy_about_handler_new ();
   webkit_web_context_register_uri_scheme (priv->web_context,
                                           EPHY_ABOUT_SCHEME,
                                           (WebKitURISchemeRequestCallback)about_request_cb,
@@ -704,7 +713,7 @@ ephy_embed_shell_startup (GApplication* application)
 static void
 ephy_embed_shell_shutdown (GApplication* application)
 {
-  EphyEmbedShellPrivate *priv = EPHY_EMBED_SHELL (application)->priv;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (application));
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->shutdown (application);
 
@@ -730,11 +739,11 @@ ephy_embed_shell_set_property (GObject *object,
                                const GValue *value,
                                GParamSpec *pspec)
 {
-  EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (object);
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
 
   switch (prop_id) {
   case PROP_MODE:
-    embed_shell->priv->mode = g_value_get_enum (value);
+    priv->mode = g_value_get_enum (value);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -747,11 +756,11 @@ ephy_embed_shell_get_property (GObject *object,
                                GValue *value,
                                GParamSpec *pspec)
 {
-  EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (object);
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
 
   switch (prop_id) {
   case PROP_MODE:
-    g_value_set_enum (value, embed_shell->priv->mode);
+    g_value_set_enum (value, priv->mode);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -762,28 +771,24 @@ static void
 ephy_embed_shell_constructed (GObject *object)
 {
   EphyEmbedShell *embed_shell;
+  EphyEmbedShellPrivate *priv;
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->constructed (object);
 
   embed_shell = EPHY_EMBED_SHELL (object);
+  priv = ephy_embed_shell_get_instance_private (embed_shell);
   /* These do not run the EmbedShell application instance, so make sure that
      there is a web context and a user content manager for them. */
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_TEST ||
       ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER) {
     ephy_embed_shell_create_web_context (embed_shell);
-    embed_shell->priv->user_content = webkit_user_content_manager_new ();
+    priv->user_content = webkit_user_content_manager_new ();
   }
-
-  g_signal_connect_object (ephy_snapshot_service_get_default (),
-                           "snapshot-saved", G_CALLBACK (snapshot_saved_cb),
-                           embed_shell, 0);
 }
 
 static void
 ephy_embed_shell_init (EphyEmbedShell *shell)
 {
-  shell->priv = EPHY_EMBED_SHELL_GET_PRIVATE (shell);
-
   /* globally accessible singleton */
   g_assert (embed_shell == NULL);
   embed_shell = shell;
@@ -809,8 +814,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                        "The  global mode for this instance.",
                        EPHY_TYPE_EMBED_SHELL_MODE,
                        EPHY_EMBED_SHELL_MODE_BROWSER,
-                       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
-                       G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT_ONLY);
+                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
   
   g_object_class_install_properties (object_class,
                                      N_PROPERTIES,
@@ -829,8 +833,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                   EPHY_TYPE_EMBED_SHELL,
                   G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (EphyEmbedShellClass, prepare_close),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
 /**
@@ -845,8 +848,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
                   EPHY_TYPE_EMBED_SHELL,
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (EphyEmbedShellClass, restored_window),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE,
                   0);
 
@@ -863,8 +865,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
     g_signal_new ("web-view-created",
                   EPHY_TYPE_EMBED_SHELL,
                   G_SIGNAL_RUN_LAST,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   EPHY_TYPE_WEB_VIEW);
 
@@ -880,8 +881,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
     g_signal_new ("page-created",
                   EPHY_TYPE_EMBED_SHELL,
                   G_SIGNAL_RUN_FIRST,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_generic,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 2,
                   G_TYPE_UINT64,
                   EPHY_TYPE_WEB_EXTENSION_PROXY);
@@ -898,8 +898,7 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
     g_signal_new ("allow-tls-certificate",
                   EPHY_TYPE_EMBED_SHELL,
                   G_SIGNAL_RUN_FIRST,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_generic,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   G_TYPE_UINT64);
 
@@ -919,15 +918,12 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
     g_signal_new ("form-auth-data-save-requested",
                   EPHY_TYPE_EMBED_SHELL,
                   G_SIGNAL_RUN_FIRST,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_generic,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 4,
                   G_TYPE_UINT,
                   G_TYPE_UINT64,
                   G_TYPE_STRING,
                   G_TYPE_STRING);
-
-  g_type_class_add_private (object_class, sizeof (EphyEmbedShellPrivate));
 }
 
 /**
@@ -947,11 +943,10 @@ void
 ephy_embed_shell_set_page_setup (EphyEmbedShell *shell,
                                  GtkPageSetup *page_setup)
 {
-  EphyEmbedShellPrivate *priv;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   char *path;
 
   g_return_if_fail (EPHY_IS_EMBED_SHELL (shell));
-  priv = shell->priv;
 
   if (page_setup != NULL)
     g_object_ref (page_setup);
@@ -976,10 +971,9 @@ ephy_embed_shell_set_page_setup (EphyEmbedShell *shell,
 GtkPageSetup *
 ephy_embed_shell_get_page_setup (EphyEmbedShell *shell)
 {
-  EphyEmbedShellPrivate *priv;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
   g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
-  priv = shell->priv;
 
   if (priv->page_setup == NULL) {
     GError *error = NULL;
@@ -1012,11 +1006,10 @@ void
 ephy_embed_shell_set_print_settings (EphyEmbedShell *shell,
                                      GtkPrintSettings *settings)
 {
-  EphyEmbedShellPrivate *priv;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
   char *path;
 
   g_return_if_fail (EPHY_IS_EMBED_SHELL (shell));
-  priv = shell->priv;
 
   if (settings != NULL)
     g_object_ref (settings);
@@ -1042,10 +1035,9 @@ ephy_embed_shell_set_print_settings (EphyEmbedShell *shell,
 GtkPrintSettings *
 ephy_embed_shell_get_print_settings (EphyEmbedShell *shell)
 {
-  EphyEmbedShellPrivate *priv;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
   g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
-  priv = shell->priv;
 
   if (priv->print_settings == NULL) {
     GError *error = NULL;
@@ -1075,9 +1067,11 @@ ephy_embed_shell_get_print_settings (EphyEmbedShell *shell)
 EphyEmbedShellMode
 ephy_embed_shell_get_mode (EphyEmbedShell *shell)
 {
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
   g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), EPHY_EMBED_SHELL_MODE_BROWSER);
   
-  return shell->priv->mode;
+  return priv->mode;
 }
 
 /**
@@ -1133,17 +1127,33 @@ ephy_embed_shell_launch_handler (EphyEmbedShell *shell,
 void
 ephy_embed_shell_clear_cache (EphyEmbedShell *shell)
 {
-  webkit_web_context_clear_cache (shell->priv->web_context);
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  webkit_web_context_clear_cache (priv->web_context);
 }
 
 WebKitUserContentManager *
 ephy_embed_shell_get_user_content_manager (EphyEmbedShell *shell)
 {
-  return shell->priv->user_content;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  return priv->user_content;
 }
 
 WebKitWebContext *
 ephy_embed_shell_get_web_context (EphyEmbedShell *shell)
 {
-  return shell->priv->web_context;
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  return priv->web_context;
+}
+
+EphyDownloadsManager *
+ephy_embed_shell_get_downloads_manager (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  if (!priv->downloads_manager)
+    priv->downloads_manager = EPHY_DOWNLOADS_MANAGER (g_object_new (EPHY_TYPE_DOWNLOADS_MANAGER, NULL));
+  return priv->downloads_manager;
 }
