@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
@@ -27,19 +25,13 @@
 #define EPHY_EMBED_SHELL_H
 
 #include <webkit2/webkit2.h>
+#include "ephy-downloads-manager.h"
 
 G_BEGIN_DECLS
 
-#define EPHY_TYPE_EMBED_SHELL   (ephy_embed_shell_get_type ())
-#define EPHY_EMBED_SHELL(o)   (G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_EMBED_SHELL, EphyEmbedShell))
-#define EPHY_EMBED_SHELL_CLASS(k) (G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellClass))
-#define EPHY_IS_EMBED_SHELL(o)    (G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_EMBED_SHELL))
-#define EPHY_IS_EMBED_SHELL_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED_SHELL))
-#define EPHY_EMBED_SHELL_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellClass))
+#define EPHY_TYPE_EMBED_SHELL (ephy_embed_shell_get_type ())
 
-typedef struct _EphyEmbedShellClass EphyEmbedShellClass;
-typedef struct _EphyEmbedShell    EphyEmbedShell;
-typedef struct _EphyEmbedShellPrivate EphyEmbedShellPrivate;
+G_DECLARE_DERIVABLE_TYPE (EphyEmbedShell, ephy_embed_shell, EPHY, EMBED_SHELL, GtkApplication)
 
 typedef enum
 {
@@ -48,20 +40,13 @@ typedef enum
   EPHY_EMBED_SHELL_MODE_PRIVATE,
   EPHY_EMBED_SHELL_MODE_INCOGNITO,
   EPHY_EMBED_SHELL_MODE_APPLICATION,
-  EPHY_EMBED_SHELL_MODE_TEST
+  EPHY_EMBED_SHELL_MODE_TEST,
+  EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER
 } EphyEmbedShellMode;
 
 #define EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE(mode) \
   (mode == EPHY_EMBED_SHELL_MODE_PRIVATE || mode == EPHY_EMBED_SHELL_MODE_INCOGNITO)
 
-struct _EphyEmbedShell
-{
-  GtkApplication parent;
-
-  /*< private >*/
-  EphyEmbedShellPrivate *priv;
-};
-
 struct _EphyEmbedShellClass
 {
   GtkApplicationClass parent_class;
@@ -71,7 +56,6 @@ struct _EphyEmbedShellClass
   void    (* restored_window)  (EphyEmbedShell *shell);
 };
 
-GType              ephy_embed_shell_get_type                   (void);
 EphyEmbedShell    *ephy_embed_shell_get_default                (void);
 WebKitWebContext  *ephy_embed_shell_get_web_context            (EphyEmbedShell   *shell);
 GObject           *ephy_embed_shell_get_global_history_service (EphyEmbedShell   *shell);
@@ -90,11 +74,12 @@ gboolean           ephy_embed_shell_launch_handler             (EphyEmbedShell
                                                                 const char       *mime_type,
                                                                 guint32           user_time);
 void               ephy_embed_shell_clear_cache                (EphyEmbedShell   *shell);
-void               ephy_embed_shell_set_thumbanil_path         (EphyEmbedShell   *shell,
+void               ephy_embed_shell_set_thumbnail_path         (EphyEmbedShell   *shell,
                                                                 const char       *url,
                                                                 time_t            mtime,
                                                                 const char       *path);
 WebKitUserContentManager *ephy_embed_shell_get_user_content_manager (EphyEmbedShell *shell);
+EphyDownloadsManager     *ephy_embed_shell_get_downloads_manager    (EphyEmbedShell *shell);
 
 G_END_DECLS
 
