@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -27,23 +25,26 @@
 #include <webkitdom/WebKitDOMElementUnstable.h>
 #include <webkitdom/WebKitDOMDOMTokenList.h>
 
-struct _EphyWebOverviewPrivate
+struct _EphyWebOverview
 {
+  GObject parent_instance;
+
   WebKitWebPage *web_page;
   EphyWebOverviewModel *model;
   GList *items;
-
-  GHashTable *delayed_thumbnail_changes;
 };
 
+G_DEFINE_TYPE (EphyWebOverview, ephy_web_overview, G_TYPE_OBJECT)
+
 enum
 {
   PROP_0,
   PROP_WEB_PAGE,
   PROP_MODEL,
+  LAST_PROP
 };
 
-G_DEFINE_TYPE (EphyWebOverview, ephy_web_overview, G_TYPE_OBJECT)
+static GParamSpec *obj_properties[LAST_PROP];
 
 typedef struct {
   char *url;
@@ -128,7 +129,7 @@ ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
 
   urls = ephy_web_overview_model_get_urls (model);
 
-  items = overview->priv->items;
+  items = overview->items;
   for (l = urls; l; l = g_list_next (l)) {
     EphyWebOverviewModelItem *url = (EphyWebOverviewModelItem *)l->data;
     const char *thumbnail_path;
@@ -144,11 +145,7 @@ ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
       item->url = g_strdup (url->url);
 
       class_list = webkit_dom_element_get_class_list (webkit_dom_node_get_parent_element (WEBKIT_DOM_NODE (item->anchor)));
-#if WEBKIT_CHECK_VERSION(2, 13, 0)
-      if (class_list && webkit_dom_dom_token_list_contains (class_list, "overview-removed"))
-#else
       if (class_list && webkit_dom_dom_token_list_contains (class_list, "overview-removed", NULL))
-#endif
         webkit_dom_dom_token_list_remove (class_list, "overview-removed", NULL);
 
       webkit_dom_element_set_attribute (item->anchor, "href", url->url, NULL);
@@ -172,7 +169,7 @@ ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
       item = g_slice_new0 (OverviewItem);
       item->url = g_strdup (url->url);
 
-      document = webkit_web_page_get_dom_document (overview->priv->web_page);
+      document = webkit_web_page_get_dom_document (overview->web_page);
       item_list = webkit_dom_document_get_element_by_id (document, "overview-item-list");
 
       new_node = WEBKIT_DOM_NODE (webkit_dom_document_create_element (document, "LI", NULL));
@@ -205,7 +202,7 @@ ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
       webkit_dom_node_set_text_content (new_node, url->title, NULL);
       webkit_dom_node_append_child (WEBKIT_DOM_NODE (anchor), new_node, NULL);
 
-      overview->priv->items = g_list_append (overview->priv->items, item);
+      overview->items = g_list_append (overview->items, item);
     }
 
     items = g_list_next (items);
@@ -221,34 +218,12 @@ ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
     webkit_dom_node_remove_child (webkit_dom_node_get_parent_node (anchor), anchor, NULL);
 
     overview_item_free (item);
-    overview->priv->items = g_list_delete_link (overview->priv->items, items);
+    overview->items = g_list_delete_link (overview->items, items);
 
     items = next;
   }
 }
 
-static gboolean
-apply_delayed_thumbnail_change (gpointer key,
-                                gpointer value,
-                                gpointer user_data)
-{
-  EphyWebOverview *overview = EPHY_WEB_OVERVIEW (user_data);
-  const char *url = key;
-  const char *path = value;
-  GList *l;
-
-  for (l = overview->priv->items; l; l = g_list_next (l)) {
-    OverviewItem *item = (OverviewItem *)l->data;
-
-    if (g_strcmp0 (item->url, url) == 0) {
-      update_thumbnail_element_style (item->thumbnail, path);
-      return TRUE;
-    }
-  }
-
-  return FALSE;
-}
-
 static void
 ephy_web_overview_model_thumbnail_changed (EphyWebOverviewModel *model,
                                            const char *url,
@@ -257,30 +232,12 @@ ephy_web_overview_model_thumbnail_changed (EphyWebOverviewModel *model,
 {
   GList *l;
 
-  for (l = overview->priv->items; l; l = g_list_next (l)) {
+  for (l = overview->items; l; l = g_list_next (l)) {
     OverviewItem *item = (OverviewItem *)l->data;
 
-    if (g_strcmp0 (item->url, url) == 0) {
+    if (g_strcmp0 (item->url, url) == 0)
       update_thumbnail_element_style (item->thumbnail, path);
-      return;
-    }
   }
-
-  if (!overview->priv->delayed_thumbnail_changes) {
-    overview->priv->delayed_thumbnail_changes = g_hash_table_new_full (g_str_hash,
-                                                                       g_str_equal,
-                                                                       g_free,
-                                                                       g_free);
-  }
-
-  /* We got the thumbnail change request before document-loaded. Save the
-   * request, else we will wind up with an overview showing placeholder icons.
-   * This isn't needed for title and URL changes because EphyAboutHandler is
-   * sure to have those right when creating the overview HTML. But thumbnail
-   * changes can arrive delayed if the snapshot service does not have the right
-   * snapshot on demand.
-   */
-  g_hash_table_insert (overview->priv->delayed_thumbnail_changes, g_strdup (url), g_strdup (path));
 }
 
 static void
@@ -291,7 +248,7 @@ ephy_web_overview_model_title_changed (EphyWebOverviewModel *model,
 {
   GList *l;
 
-  for (l = overview->priv->items; l; l = g_list_next (l)) {
+  for (l = overview->items; l; l = g_list_next (l)) {
     OverviewItem *item = (OverviewItem *)l->data;
 
     if (g_strcmp0 (item->url, url) != 0)
@@ -327,16 +284,16 @@ ephy_web_overview_update_thumbnail_in_model_from_element (EphyWebOverview *overv
     p = g_strrstr (thumbnail_path, ")");
     if (p) {
       thumbnail_path = g_strndup (thumbnail_path, strlen (thumbnail_path) - strlen (p));
-      g_signal_handlers_block_by_func (overview->priv->model, G_CALLBACK (ephy_web_overview_model_thumbnail_changed), overview);
-      ephy_web_overview_model_set_url_thumbnail (overview->priv->model, url, thumbnail_path);
-      g_signal_handlers_unblock_by_func (overview->priv->model, G_CALLBACK (ephy_web_overview_model_thumbnail_changed), overview);
+      g_signal_handlers_block_by_func (overview->model, G_CALLBACK (ephy_web_overview_model_thumbnail_changed), overview);
+      ephy_web_overview_model_set_url_thumbnail (overview->model, url, thumbnail_path);
+      g_signal_handlers_unblock_by_func (overview->model, G_CALLBACK (ephy_web_overview_model_thumbnail_changed), overview);
       g_free (thumbnail_path);
     }
   } else {
     const char *path;
 
     /* Check whether the model was updated while the overview was loading */
-    path = ephy_web_overview_model_get_url_thumbnail (overview->priv->model, url);
+    path = ephy_web_overview_model_get_url_thumbnail (overview->model, url);
     if (path)
       update_thumbnail_element_style (thumbnail, path);
   }
@@ -367,19 +324,12 @@ ephy_web_overview_document_loaded (WebKitWebPage *web_page,
        */
       ephy_web_overview_update_thumbnail_in_model_from_element (overview, item->url, item->thumbnail);
 
-      overview->priv->items = g_list_prepend (overview->priv->items, item);
+      overview->items = g_list_prepend (overview->items, item);
     }
     g_free (class);
   }
   g_object_unref (nodes);
-  overview->priv->items = g_list_reverse (overview->priv->items);
-
-  if (overview->priv->delayed_thumbnail_changes) {
-    g_hash_table_foreach_remove (overview->priv->delayed_thumbnail_changes,
-                                 apply_delayed_thumbnail_change,
-                                 overview);
-    g_clear_pointer (&overview->priv->delayed_thumbnail_changes, g_hash_table_unref);
-  }
+  overview->items = g_list_reverse (overview->items);
 }
 
 static void
@@ -393,10 +343,10 @@ ephy_web_overview_set_property (GObject *object,
   switch (prop_id)
   {
   case PROP_WEB_PAGE:
-    overview->priv->web_page = g_value_get_object (value);
+    overview->web_page = g_value_get_object (value);
     break;
   case PROP_MODEL:
-    overview->priv->model = g_value_get_object (value);
+    overview->model = g_value_get_object (value);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -409,13 +359,11 @@ ephy_web_overview_dispose (GObject *object)
 {
   EphyWebOverview *overview = EPHY_WEB_OVERVIEW (object);
 
-  if (overview->priv->items) {
-    g_list_free_full (overview->priv->items, (GDestroyNotify)overview_item_free);
-    overview->priv->items = NULL;
+  if (overview->items) {
+    g_list_free_full (overview->items, (GDestroyNotify)overview_item_free);
+    overview->items = NULL;
   }
 
-  g_clear_pointer (&overview->priv->delayed_thumbnail_changes, g_hash_table_unref);
-
   G_OBJECT_CLASS (ephy_web_overview_parent_class)->dispose (object);
 }
 
@@ -426,17 +374,17 @@ ephy_web_overview_constructed (GObject *object)
 
   G_OBJECT_CLASS (ephy_web_overview_parent_class)->constructed (object);
 
-  g_signal_connect_object (overview->priv->web_page, "document-loaded",
+  g_signal_connect_object (overview->web_page, "document-loaded",
                            G_CALLBACK (ephy_web_overview_document_loaded),
                            overview, 0);
 
-  g_signal_connect_object (overview->priv->model, "urls-changed",
+  g_signal_connect_object (overview->model, "urls-changed",
                            G_CALLBACK (ephy_web_overview_model_urls_changed),
                            overview, 0);
-  g_signal_connect_object (overview->priv->model, "thumbnail-changed",
+  g_signal_connect_object (overview->model, "thumbnail-changed",
                            G_CALLBACK (ephy_web_overview_model_thumbnail_changed),
                            overview, 0);
-  g_signal_connect_object (overview->priv->model, "title-changed",
+  g_signal_connect_object (overview->model, "title-changed",
                            G_CALLBACK (ephy_web_overview_model_title_changed),
                            overview, 0);
 }
@@ -450,32 +398,26 @@ ephy_web_overview_class_init (EphyWebOverviewClass *klass)
   object_class->constructed = ephy_web_overview_constructed;
   object_class->set_property = ephy_web_overview_set_property;
 
-  g_object_class_install_property (object_class,
-                                   PROP_WEB_PAGE,
-                                   g_param_spec_object ("web-page",
-                                                        "WebPage",
-                                                        "The overview WebPage",
-                                                        WEBKIT_TYPE_WEB_PAGE,
-                                                        G_PARAM_WRITABLE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
-  g_object_class_install_property (object_class,
-                                   PROP_MODEL,
-                                   g_param_spec_object ("model",
-                                                        "Model",
-                                                        "The overview model",
-                                                        EPHY_TYPE_WEB_OVERVIEW_MODEL,
-                                                        G_PARAM_WRITABLE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_WEB_PAGE] =
+    g_param_spec_object ("web-page",
+                         "WebPage",
+                         "The overview WebPage",
+                         WEBKIT_TYPE_WEB_PAGE,
+                         G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
-  g_type_class_add_private (object_class, sizeof(EphyWebOverviewPrivate));
+  obj_properties[PROP_MODEL] =
+    g_param_spec_object ("model",
+                         "Model",
+                         "The overview model",
+                         EPHY_TYPE_WEB_OVERVIEW_MODEL,
+                         G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 }
 
 static void
 ephy_web_overview_init (EphyWebOverview *overview)
 {
-  overview->priv = G_TYPE_INSTANCE_GET_PRIVATE (overview, EPHY_TYPE_WEB_OVERVIEW, EphyWebOverviewPrivate);
 }
 
 EphyWebOverview *
