@@ -16,9 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -85,11 +83,7 @@ ephy_file_tmp_dir (void)
 					    "-XXXXXX", NULL);
 		full_name = g_build_filename (g_get_tmp_dir (), partial_name,
 					      NULL);
-#ifdef HAVE_MKDTEMP
 		tmp_dir = mkdtemp (full_name);
-#else
-#error no mkdtemp implementation
-#endif
 		g_free (partial_name);
 
 		if (tmp_dir == NULL)
@@ -290,6 +284,19 @@ ephy_dot_dir_is_default (void)
 	return is_default_dot_dir;
 }
 
+/**
+ * ephy_default_dot_dir:
+ *
+ * Get the path to the default dot directory found in ~/.config
+ *
+ * Returns: a new allocated string, free with g_free() when done.
+ */
+char *
+ephy_default_dot_dir (void)
+{
+	return g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
+}
+
 /**
  * ephy_file_helpers_init:
  * @profile_dir: directory to use as Epiphany's profile
@@ -343,9 +350,7 @@ ephy_file_helpers_init (const char *profile_dir,
 	}
 	else
 	{
-		dot_dir = g_build_filename (g_get_user_config_dir (),
-					    "epiphany",
-					    NULL);
+		dot_dir = ephy_default_dot_dir ();
 		is_default_dot_dir = TRUE;
 	}
 
@@ -473,11 +478,14 @@ ephy_ensure_dir_exists (const char *dir,
 	{
 		if (g_mkdir_with_parents (dir, 488) == 0)
 		{
-			/* We need to set the .migrated file to the
-			 * current profile migration version,
-			 * otherwise the next time the browser runs
-			 * things might go awry. */
-			ephy_profile_utils_set_migration_version (EPHY_PROFILE_MIGRATION_VERSION);
+			if (dir == ephy_dot_dir ())
+			{
+				/* We need to set the .migrated file to the
+				 * current profile migration version,
+				 * otherwise the next time the browser runs
+				 * things might go awry. */
+				ephy_profile_utils_set_migration_version (EPHY_PROFILE_MIGRATION_VERSION);
+			}
 		}
 		else
 		{
@@ -829,8 +837,7 @@ ephy_file_open_uri_in_default_browser (const char *uri,
 	gdk_app_launch_context_set_screen (context, screen);
 	gdk_app_launch_context_set_timestamp (context, timestamp);
 
-	/* We assume the default application that supports HTML and URIs is the default browser */
-	appinfo = g_app_info_get_default_for_type ("text/html", TRUE);
+	appinfo = g_app_info_get_default_for_type ("x-scheme-handler/http", TRUE);
 	uris.data = (gpointer)uri;
 	uris.next = uris.prev = NULL;
 
