@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -781,7 +779,6 @@ ephy_node_get_property_node (EphyNode *node,
 	EphyNode *retval;
 
 	g_return_val_if_fail (EPHY_IS_NODE (node), NULL);
-	g_return_val_if_fail (property_id >= 0, NULL);
 
 	if (property_id >= node->properties->len) {
 		return NULL;
@@ -1268,7 +1265,6 @@ ephy_node_get_nth_child (EphyNode *node,
 	EphyNode *ret;
 
 	g_return_val_if_fail (EPHY_IS_NODE (node), NULL);
-	g_return_val_if_fail (n >= 0, NULL);
 
 	if (n < node->children->len) {
 		ret = g_ptr_array_index (node->children, n);
