@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _EPHY_SNAPSHOT_SERVICE_H
@@ -105,8 +103,27 @@ char                *ephy_snapshot_service_save_snapshot_finish        (EphySnap
                                                                         GAsyncResult *result,
                                                                         GError **error);
 
-GdkPixbuf           *ephy_snapshot_service_prepare_snapshot            (cairo_surface_t *surface,
-                                                                        cairo_surface_t *favicon);
+const char          *ephy_snapshot_service_lookup_snapshot_path        (EphySnapshotService *service,
+                                                                        const char *url);
+
+void             ephy_snapshot_service_get_snapshot_path_for_url_async (EphySnapshotService *service,
+                                                                        const char *url,
+                                                                        const time_t mtime,
+                                                                        GCancellable *cancellable,
+                                                                        GAsyncReadyCallback callback,
+                                                                        gpointer user_data);
+char           *ephy_snapshot_service_get_snapshot_path_for_url_finish (EphySnapshotService *service,
+                                                                        GAsyncResult *result,
+                                                                        GError **error);
+void           ephy_snapshot_service_get_snapshot_path_async           (EphySnapshotService *service,
+                                                                        WebKitWebView *web_view,
+                                                                        const time_t mtime,
+                                                                        GCancellable *cancellable,
+                                                                        GAsyncReadyCallback callback,
+                                                                        gpointer user_data);
+char                *ephy_snapshot_service_get_snapshot_path_finish    (EphySnapshotService *service,
+                                                                        GAsyncResult *result,
+                                                                        GError **error);
 
 G_END_DECLS
 
