@@ -12,8 +12,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef EPHY_SQLITE_CONNECTION_H
@@ -22,8 +21,6 @@
 #include <glib-object.h>
 #include "ephy-sqlite-statement.h"
 
-#include <sqlite3.h>
-
 G_BEGIN_DECLS
 
 /* convenience macros */
@@ -51,12 +48,7 @@ struct _EphySQLiteConnectionClass {
 
 GType                   ephy_sqlite_connection_get_type                (void);
 
-typedef enum {
-  EPHY_SQLITE_CONNECTION_MODE_READ_ONLY,
-  EPHY_SQLITE_CONNECTION_MODE_READWRITE
-} EphySQLiteConnectionMode;
-
-EphySQLiteConnection *  ephy_sqlite_connection_new                     (EphySQLiteConnectionMode mode);
+EphySQLiteConnection *  ephy_sqlite_connection_new                     (void);
 
 gboolean                ephy_sqlite_connection_open                    (EphySQLiteConnection *self, const gchar *filename, GError **error);
 void                    ephy_sqlite_connection_close                   (EphySQLiteConnection *self);
@@ -73,10 +65,6 @@ gboolean                ephy_sqlite_connection_commit_transaction      (EphySQLi
 
 gboolean                ephy_sqlite_connection_table_exists            (EphySQLiteConnection *self, const char *table_name);
 
-GQuark                  ephy_sqlite_error_quark                        (void);
-
-#define EPHY_SQLITE_ERROR ephy_sqlite_error_quark ()
-
 G_END_DECLS
 
 #endif /* EPHY_SQLITE_CONNECTION_H */
