@@ -12,8 +12,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -144,39 +143,6 @@ ephy_history_service_dispose (GObject *self)
     g_source_remove (priv->queue_urls_visited_id);
     priv->queue_urls_visited_id = 0;
   }
-
-  G_OBJECT_CLASS (ephy_history_service_parent_class)->dispose (self);
-}
-
-static void
-ephy_history_service_constructed (GObject *object)
-{
-  EphyHistoryService *self = EPHY_HISTORY_SERVICE (object);
-  EphyHistoryServicePrivate *priv = self->priv;
-
-  G_OBJECT_CLASS (ephy_history_service_parent_class)->constructed (object);
-
-  priv->queue = g_async_queue_new ();
-
-  /* This value is checked in several functions to verify that they are only
-   * ever run on the history thread. Accordingly, we'd better be sure it's set
-   * before it is checked for the first time. That requires a lock here. */
-  g_mutex_lock (&priv->history_thread_mutex);
-  priv->history_thread = g_thread_new ("EphyHistoryService", (GThreadFunc)run_history_service_thread, self);
-
-  /* Additionally, make sure the SQLite connection has really been opened before
-   * returning. We need this so that we can test that using a read-only service
-   * at the same time as a read/write service does not cause the read/write
-   * service to break. This delay is required because we need to be sure the
-   * read/write service has completed initialization before attempting to open
-   * the read-only service, or initializing the read-only service will fail.
-   * This isn't needed except in test mode, because only tests might run
-   * multiple history services, but it's harmless and cleaner to do always.
-   */
-  while (!priv->history_thread_initialized)
-    g_cond_wait (&priv->history_thread_initialized_condition, &priv->history_thread_mutex);
-
-  g_mutex_unlock (&priv->history_thread_mutex);
 }
 
 static gboolean
@@ -221,8 +187,7 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize = ephy_history_service_finalize;
-  gobject_class->dispose = ephy_history_service_dispose;
-  gobject_class->constructed = ephy_history_service_constructed;
+  gobject_class->dispose  = ephy_history_service_dispose;
   gobject_class->get_property = ephy_history_service_get_property;
   gobject_class->set_property = ephy_history_service_set_property;
 
@@ -321,16 +286,19 @@ static void
 ephy_history_service_init (EphyHistoryService *self)
 {
   self->priv = EPHY_HISTORY_SERVICE_GET_PRIVATE (self);
+
+  self->priv->history_thread = g_thread_new ("EphyHistoryService", (GThreadFunc) run_history_service_thread, self);
+  self->priv->queue = g_async_queue_new ();
 }
 
 EphyHistoryService *
-ephy_history_service_new (const char               *history_filename,
-                          EphySQLiteConnectionMode  mode)
+ephy_history_service_new (const char *history_filename,
+                          gboolean read_only)
 {
   return EPHY_HISTORY_SERVICE (g_object_new (EPHY_TYPE_HISTORY_SERVICE,
                                              "history-filename", history_filename,
-                                             "read-only", mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY,
-                                             NULL));
+                                             "read-only", read_only,
+                                              NULL));
 }
 
 static gint
@@ -434,23 +402,12 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
 
   g_assert (priv->history_thread == g_thread_self ());
 
-  if (priv->history_database != NULL)
-    g_object_unref (priv->history_database);
-
-  priv->history_database = ephy_sqlite_connection_new (priv->read_only ? EPHY_SQLITE_CONNECTION_MODE_READ_ONLY
-                                                                       : EPHY_SQLITE_CONNECTION_MODE_READWRITE);
+  priv->history_database = ephy_sqlite_connection_new ();
   ephy_sqlite_connection_open (priv->history_database, priv->history_filename, &error);
   if (error) {
     g_object_unref (priv->history_database);
     priv->history_database = NULL;
-
-    /* Opening the database is expected to fail if it's being opened in read-
-     * only mode and does not already exist. Otherwise, this is bad. */
-    if (!priv->read_only ||
-        !g_error_matches (error, EPHY_SQLITE_ERROR, SQLITE_CANTOPEN) ||
-        g_file_test (priv->history_filename, G_FILE_TEST_EXISTS)) {
-      g_warning ("Could not open history database at %s: %s", priv->history_filename, error->message);
-    }
+    g_warning ("Could not open history database at %s: %s", priv->history_filename, error->message);
     g_error_free (error);
     return FALSE;
   }
@@ -547,23 +504,10 @@ run_history_service_thread (EphyHistoryService *self)
 {
   EphyHistoryServicePrivate *priv = self->priv;
   EphyHistoryServiceMessage *message;
-  gboolean success;
 
-  /* Note that self->history_thread is only written once, and that's guaranteed
-   * to have occurred before we enter this critical section due to this mutex.
-   * Accordingly, we do not need to use the mutex when performing these
-   * assertions in other functions.
-   */
-  g_mutex_lock (&priv->history_thread_mutex);
   g_assert (priv->history_thread == g_thread_self ());
 
-  success = ephy_history_service_open_database_connections (self);
-
-  priv->history_thread_initialized = TRUE;
-  g_cond_signal (&priv->history_thread_initialized_condition);
-  g_mutex_unlock (&priv->history_thread_mutex);
-
-  if (!success)
+  if (ephy_history_service_open_database_connections (self) == FALSE)
     return NULL;
 
   do {
