@@ -14,8 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef EPHY_HISTORY_SERVICE_H
@@ -23,9 +22,7 @@
 
 #include <glib-object.h>
 #include <gio/gio.h>
-
 #include "ephy-history-types.h"
-#include "ephy-sqlite-connection.h"
 
 G_BEGIN_DECLS
 
@@ -60,7 +57,7 @@ struct _EphyHistoryServiceClass {
 };
 
 GType                    ephy_history_service_get_type                (void);
-EphyHistoryService *     ephy_history_service_new                     (const char *history_filename, EphySQLiteConnectionMode mode);
+EphyHistoryService *     ephy_history_service_new                     (const char *history_filename, gboolean read_only);
 
 void                     ephy_history_service_add_visit               (EphyHistoryService *self, EphyHistoryPageVisit *visit, GCancellable *cancellable, EphyHistoryJobCallback callback, gpointer user_data);
 void                     ephy_history_service_add_visits              (EphyHistoryService *self, GList *visits, GCancellable *cancellable, EphyHistoryJobCallback callback, gpointer user_data);
