@@ -12,14 +12,14 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "ephy-certificate-dialog.h"
 
+#include "ephy-lib-type-builtins.h"
+
 #define GCR_API_SUBJECT_TO_CHANGE
 #include <gcr/gcr.h>
 #include <glib/gi18n.h>
@@ -37,6 +37,7 @@ enum
   PROP_0,
   PROP_ADDRESS,
   PROP_CERTIFICATE,
+  PROP_SECURITY_LEVEL,
   PROP_TLS_ERRORS
 };
 
@@ -45,6 +46,8 @@ struct _EphyCertificateDialogPrivate
   GtkWidget *icon;
   GtkWidget *title;
   GtkWidget *text;
+  GTlsCertificateFlags tls_errors;
+  EphySecurityLevel security_level;
 };
 
 G_DEFINE_TYPE (EphyCertificateDialog, ephy_certificate_dialog, GTK_TYPE_DIALOG)
@@ -89,7 +92,7 @@ get_error_messages_from_tls_errors (GTlsCertificateFlags tls_errors)
   char *retval;
 
   if (tls_errors & G_TLS_CERTIFICATE_BAD_IDENTITY)
-    g_ptr_array_add (errors, _("The certificate does not match the expected identity"));
+    g_ptr_array_add (errors, _("The certificate does not match this website"));
 
   if (tls_errors & G_TLS_CERTIFICATE_EXPIRED)
     g_ptr_array_add (errors, _("The certificate has expired"));
@@ -131,34 +134,49 @@ get_error_messages_from_tls_errors (GTlsCertificateFlags tls_errors)
 }
 
 static void
-ephy_certificate_dialog_set_tls_errors (EphyCertificateDialog *dialog,
-                                        GTlsCertificateFlags tls_errors)
+ephy_certificate_dialog_constructed (GObject *object)
 {
+  EphyCertificateDialog *dialog = EPHY_CERTIFICATE_DIALOG (object);
   EphyCertificateDialogPrivate *priv = dialog->priv;
   GIcon *icon;
+  const char *icon_name;
   char *markup;
 
-  icon = tls_errors == 0 ?
-    g_themed_icon_new_with_default_fallbacks ("channel-secure-symbolic") :
-    g_themed_icon_new_with_default_fallbacks ("channel-insecure-symbolic");
-  gtk_image_set_from_gicon (GTK_IMAGE (priv->icon), icon, GTK_ICON_SIZE_DIALOG);
-  g_object_unref (icon);
+  G_OBJECT_CLASS (ephy_certificate_dialog_parent_class)->constructed (object);
+
+  icon_name = ephy_security_level_to_icon_name (priv->security_level);
+  if (icon_name) {
+    icon = g_themed_icon_new_with_default_fallbacks (icon_name);
+    gtk_image_set_from_gicon (GTK_IMAGE (priv->icon), icon, GTK_ICON_SIZE_DIALOG);
+    g_object_unref (icon);
+  }
 
   markup = g_strdup_printf ("<span weight=\"bold\" size=\"large\">%s</span>",
-			    tls_errors == 0 ?
-			    _("The identity of this website has been verified") :
-			    _("The identity of this website has not been verified"));
+			    priv->tls_errors == 0 ?
+			    _("The identity of this website has been verified.") :
+			    _("The identity of this website has not been verified."));
   gtk_label_set_markup (GTK_LABEL (priv->title), markup);
   g_free (markup);
 
-  if (tls_errors) {
-    char *text = get_error_messages_from_tls_errors (tls_errors);
-
+  if (priv->tls_errors) {
+    char *text = get_error_messages_from_tls_errors (priv->tls_errors);
     gtk_label_set_text (GTK_LABEL (priv->text), text);
     g_free (text);
-
-    gtk_widget_show (priv->text);
+  } else {
+    switch (priv->security_level) {
+    case EPHY_SECURITY_LEVEL_STRONG_SECURITY:
+      /* Message on certificte dialog ertificate dialog */
+      gtk_label_set_text (GTK_LABEL (priv->text), _("No problems have been detected with your connection."));
+      break;
+    case EPHY_SECURITY_LEVEL_MIXED_CONTENT:
+      gtk_label_set_text (GTK_LABEL (priv->text), _("This certificate is valid. However, "
+                                                    "resources on this page were sent insecurely."));
+      break;
+    default:
+      g_assert_not_reached ();
+    }
   }
+  gtk_widget_show (priv->text);
 }
 
 static void
@@ -168,6 +186,7 @@ ephy_certificate_dialog_set_property (GObject *object,
                                       GParamSpec *pspec)
 {
   EphyCertificateDialog *dialog = EPHY_CERTIFICATE_DIALOG (object);
+  EphyCertificateDialogPrivate *priv = dialog->priv;
 
   switch (prop_id) {
   case PROP_ADDRESS:
@@ -176,8 +195,11 @@ ephy_certificate_dialog_set_property (GObject *object,
   case PROP_CERTIFICATE:
     ephy_certificate_dialog_set_certificate (dialog, g_value_get_object (value));
     break;
+  case PROP_SECURITY_LEVEL:
+    priv->security_level = g_value_get_enum (value);
+    break;
   case PROP_TLS_ERRORS:
-    ephy_certificate_dialog_set_tls_errors (dialog, g_value_get_flags (value));
+    priv->tls_errors = g_value_get_flags (value);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -189,6 +211,7 @@ ephy_certificate_dialog_class_init (EphyCertificateDialogClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->constructed = ephy_certificate_dialog_constructed;
   object_class->set_property = ephy_certificate_dialog_set_property;
 
   /**
@@ -225,6 +248,24 @@ ephy_certificate_dialog_class_init (EphyCertificateDialogClass *klass)
 							G_PARAM_STATIC_NICK |
 							G_PARAM_STATIC_BLURB));
 
+  /**
+   * EphySecurityLevel:security-level:
+   *
+   * Indicates whether something is wrong with the connection.
+   */
+  g_object_class_install_property (object_class,
+				   PROP_SECURITY_LEVEL,
+				   g_param_spec_enum ("security-level",
+						      "Security Level",
+						      "Indicates whether something is wrong with the connection",
+						      EPHY_TYPE_SECURITY_LEVEL,
+						      EPHY_SECURITY_LEVEL_TO_BE_DETERMINED,
+						      G_PARAM_WRITABLE |
+						      G_PARAM_CONSTRUCT_ONLY |
+						      G_PARAM_STATIC_NAME |
+						      G_PARAM_STATIC_NICK |
+						      G_PARAM_STATIC_BLURB));
+
   /**
    * EphyCertificateDialog:tls-errors:
    *
@@ -250,7 +291,7 @@ static void
 ephy_certificate_dialog_init (EphyCertificateDialog *dialog)
 {
   GtkWidget *grid;
-  GtkWidget *content_area, *action_area;
+  GtkWidget *content_area;
   EphyCertificateDialogPrivate *priv;
 
   dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE (dialog,
@@ -258,7 +299,7 @@ ephy_certificate_dialog_init (EphyCertificateDialog *dialog)
 					      EphyCertificateDialogPrivate);
   priv = dialog->priv;
 
-  gtk_window_set_default_size (GTK_WINDOW (dialog), -1, 600);
+  gtk_window_set_default_size (GTK_WINDOW (dialog), -1, 500);
 
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 10);
   gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), TRUE);
@@ -271,7 +312,7 @@ ephy_certificate_dialog_init (EphyCertificateDialog *dialog)
   gtk_widget_set_halign (priv->icon, GTK_ALIGN_CENTER);
   gtk_widget_set_valign (priv->icon, GTK_ALIGN_START);
   gtk_grid_attach (GTK_GRID (grid), priv->icon,
-                   0, 0, 1, 1);
+                   0, 0, 1, 2);
   gtk_widget_show (priv->icon);
 
   priv->title = gtk_label_new (NULL);
@@ -300,17 +341,14 @@ ephy_certificate_dialog_init (EphyCertificateDialog *dialog)
   gtk_box_set_spacing (GTK_BOX (content_area), 14);
   gtk_box_pack_start (GTK_BOX (content_area), grid, FALSE, FALSE, 0);
   gtk_widget_show (grid);
-
-  action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
-  gtk_container_set_border_width (GTK_CONTAINER (action_area), 5);
-  gtk_box_set_spacing (GTK_BOX (action_area), 6);
 }
 
 GtkWidget *
 ephy_certificate_dialog_new (GtkWindow *parent,
                              const char *address,
                              GTlsCertificate *certificate,
-                             GTlsCertificateFlags tls_errors)
+                             GTlsCertificateFlags tls_errors,
+                             EphySecurityLevel security_level)
 {
   GtkWidget *dialog;
 
@@ -320,6 +358,7 @@ ephy_certificate_dialog_new (GtkWindow *parent,
   dialog = GTK_WIDGET (g_object_new (EPHY_TYPE_CERTIFICATE_DIALOG,
 				     "address", address,
 				     "certificate", certificate,
+				     "security-level", security_level,
 				     "tls-errors", tls_errors,
                                      "modal", TRUE,
                                      "use-header-bar", TRUE,
