@@ -16,9 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -95,11 +93,6 @@ static const GtkTargetEntry url_drag_types [] =
 
 static gboolean ephy_location_entry_reset_internal (EphyLocationEntry *, gboolean);
 
-static void textcell_data_func (GtkCellLayout *cell_layout,
-				GtkCellRenderer *cell,
-				GtkTreeModel *tree_model,
-				GtkTreeIter *iter,
-				gpointer data);
 static void extracell_data_func (GtkCellLayout *cell_layout,
 				 GtkCellRenderer *cell,
 				 GtkTreeModel *tree_model,
@@ -257,7 +250,7 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 							     "Security level",
 							     "State of the security icon",
 							     EPHY_TYPE_SECURITY_LEVEL,
-							     EPHY_SECURITY_LEVEL_NO_SECURITY,
+							     EPHY_SECURITY_LEVEL_TO_BE_DETERMINED,
 							     G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
