@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -27,6 +25,7 @@
 #include "ephy-debug.h"
 #include "ephy-dnd.h"
 #include "ephy-embed-prefs.h"
+#include "ephy-favicon-helpers.h"
 #include "ephy-gui.h"
 #include "ephy-shell.h"
 #include "ephy-string.h"
@@ -66,87 +65,90 @@ typedef struct
 
 G_DEFINE_TYPE (EphyBookmarkAction, ephy_bookmark_action, EPHY_TYPE_LINK_ACTION)
 
-#ifdef HAVE_WEBKIT2
-/* TODO: Favicons */
-#else
 static void
-favicon_loaded_cb (WebKitFaviconDatabase *database,
-		   const char *page_address,
-		   EphyBookmarkAction *action)
+favicon_changed_cb (WebKitFaviconDatabase *database,
+                    const char *page_address,
+                    const char *icon_address,
+                    EphyBookmarkAction *action)
 {
-	const char *icon;
-	char *icon_address;
+        const char *icon;
 
-	g_return_if_fail (action->priv->node != NULL);
+        icon = ephy_node_get_property_string (action->priv->node,
+                                              EPHY_NODE_BMK_PROP_ICON);
+        if (g_strcmp0 (icon, icon_address) == 0)
+        {
+                g_signal_handler_disconnect (database, action->priv->cache_handler);
+                action->priv->cache_handler = 0;
 
-	icon = ephy_node_get_property_string (action->priv->node,
-					      EPHY_NODE_BMK_PROP_ICON);
-	icon_address = webkit_favicon_database_get_favicon_uri (database, page_address);
+                g_object_notify (G_OBJECT (action), "icon");
+        }
+}
 
-	if (g_strcmp0 (icon, icon_address) == 0)
-	{
-		g_signal_handler_disconnect (database, action->priv->cache_handler);
-		action->priv->cache_handler = 0;
+static void
+async_get_favicon_pixbuf_callback (GObject *source, GAsyncResult *result, gpointer user_data)
+{
+       GtkWidget *proxy = GTK_WIDGET (user_data);
+       WebKitFaviconDatabase *database = WEBKIT_FAVICON_DATABASE (source);
+       GdkPixbuf *pixbuf = NULL;
 
-		g_object_notify (G_OBJECT (action), "icon");
-	}
+       cairo_surface_t *icon_surface = webkit_favicon_database_get_favicon_finish (database, result, NULL);
+       if (icon_surface)
+       {
+               pixbuf = ephy_pixbuf_get_from_surface_scaled (icon_surface, FAVICON_SIZE, FAVICON_SIZE);
+               cairo_surface_destroy (icon_surface);
+       }
 
-	g_free (icon_address);
+       if (pixbuf)
+       {
+               if (GTK_IS_MENU_ITEM (proxy))
+               {
+                       GtkWidget *image;
+
+                       image = gtk_image_new_from_pixbuf (pixbuf);
+                       gtk_widget_show (image);
+
+                       gtk_image_menu_item_set_image
+                         (GTK_IMAGE_MENU_ITEM (proxy), image);
+                       gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (proxy),
+                                                                  TRUE);
+               }
+               g_object_unref (pixbuf);
+       }
+
+       g_object_unref (proxy);
 }
-#endif
 
 static void
 ephy_bookmark_action_sync_icon (GtkAction *action,
 				GParamSpec *pspec,
 				GtkWidget *proxy)
 {
-#ifdef HAVE_WEBKIT2
-        /* TODO: Favicons */
-#else
 	EphyBookmarkAction *bma = EPHY_BOOKMARK_ACTION (action);
 	const char *page_location;
 	WebKitFaviconDatabase *database;
-	GdkPixbuf *pixbuf = NULL;
+        EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
 	g_return_if_fail (bma->priv->node != NULL);
 
 	page_location = ephy_node_get_property_string (bma->priv->node,
 						       EPHY_NODE_BMK_PROP_LOCATION);
 
-	database = webkit_get_favicon_database ();
+        database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (shell));
+
 	if (page_location && *page_location)
 	{
-		pixbuf = webkit_favicon_database_try_get_favicon_pixbuf (database, page_location,
-									 FAVICON_SIZE, FAVICON_SIZE);
-
-		if (pixbuf == NULL && bma->priv->cache_handler == 0)
-		{
-			bma->priv->cache_handler =
-				g_signal_connect_object
-					(database, "icon-loaded",
-					 G_CALLBACK (favicon_loaded_cb),
-					 action, 0);
-		}
+                webkit_favicon_database_get_favicon (database, page_location,
+                                                     0, async_get_favicon_pixbuf_callback,
+                                                     g_object_ref (proxy));
+                if (bma->priv->cache_handler == 0)
+                {
+	                bma->priv->cache_handler =
+			g_signal_connect_object
+				(database, "favicon-changed",
+				 G_CALLBACK (favicon_changed_cb),
+				 action, 0);
+                }
 	}
-
-	if (GTK_IS_MENU_ITEM (proxy) && pixbuf)
-	{
-		GtkWidget *image;
-
-		image = gtk_image_new_from_pixbuf (pixbuf);
-		gtk_widget_show (image);
-
-		gtk_image_menu_item_set_image
-			(GTK_IMAGE_MENU_ITEM (proxy), image);
-		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (proxy),
-							   TRUE);
-	}
-
-	if (pixbuf)
-	{
-		g_object_unref (pixbuf);
-	}
-#endif
 }
 
 void
@@ -255,7 +257,7 @@ void
 ephy_bookmark_action_updated (EphyBookmarkAction *action)
 {
 	GValue value = { 0, };
-	EphyBookmarks *bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	EphyBookmarks *bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
 	EphyNode *smart = ephy_bookmarks_get_smart_bookmarks (bookmarks);
 	EphyNode *node = action->priv->node;
 	const char *title;
@@ -384,13 +386,12 @@ ephy_bookmark_action_dispose (GObject *object)
 
 	if (priv->cache_handler != 0)
 	{
-#ifdef HAVE_WEBKIT2
-                /* TODO: Favicons */
-#else
-		WebKitFaviconDatabase *database = webkit_get_favicon_database ();
+                EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+		WebKitFaviconDatabase *database;
+
+                database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (shell));
 		g_signal_handler_disconnect (database, priv->cache_handler);
 		priv->cache_handler = 0;
-#endif
 	}
 
 	G_OBJECT_CLASS (ephy_bookmark_action_parent_class)->dispose (object);
