@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -34,6 +32,7 @@
 #include "ephy-dnd.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
+#include "ephy-uri-helpers.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -44,19 +43,31 @@ static const GtkTargetEntry dest_drag_types[] = {
   {EPHY_DND_URL_TYPE, 0, 0},
 };
 
-#define EPHY_BOOKMARK_PROPERTIES_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_BOOKMARK_PROPERTIES, EphyBookmarkPropertiesPrivate))
-
-struct _EphyBookmarkPropertiesPrivate
+struct _EphyBookmarkProperties
 {
+	GtkDialog parent_instance;
+
+	/* construct properties */
 	EphyBookmarks *bookmarks;
 	EphyNode *bookmark;
 	gboolean creating;
-	
+
+	/* counters */
 	gint duplicate_count;
 	gint duplicate_idle;
-	
-	GtkWidget *warning;
-	GtkWidget *entry;
+
+	/* from UI file */
+	GtkGrid           *grid;
+	GtkEntry          *title_entry;
+	GtkEntry          *adress_entry;
+	GtkLabel          *topics_label;
+	GtkExpander       *topics_expander;
+	GtkTreeView       *topics_tree_view;
+	GtkScrolledWindow *topics_scrolled_window;
+	GtkLabel          *warning_label;
+
+	/* model */
+	EphyTopicsPalette *topics_model;
 };
 
 enum
@@ -72,17 +83,16 @@ G_DEFINE_TYPE (EphyBookmarkProperties, ephy_bookmark_properties, GTK_TYPE_DIALOG
 static gboolean
 update_warning (EphyBookmarkProperties *properties)
 {
-	EphyBookmarkPropertiesPrivate *priv = properties->priv;
 	char *label;
 
-	priv->duplicate_idle = 0;	
-	priv->duplicate_count = ephy_bookmarks_get_similar
-	  (priv->bookmarks, priv->bookmark, NULL, NULL);
-	
+	properties->duplicate_idle = 0;
+	properties->duplicate_count = ephy_bookmarks_get_similar
+	  (properties->bookmarks, properties->bookmark, NULL, NULL);
+
         /* Translators: This string is used when counting bookmarks that
          * are similar to each other */
-	label = g_strdup_printf (ngettext("%d bookmark is similar", "%d bookmarks are similar", priv->duplicate_count), priv->duplicate_count);
-	gtk_label_set_text (GTK_LABEL (priv->warning), label);
+	label = g_strdup_printf (ngettext("%d bookmark is similar", "%d bookmarks are similar", properties->duplicate_count), properties->duplicate_count);
+	gtk_label_set_text (properties->warning_label, label);
 	g_free (label);
 
 	return FALSE;
@@ -91,16 +101,14 @@ update_warning (EphyBookmarkProperties *properties)
 static void
 update_warning_idle (EphyBookmarkProperties *properties)
 {
-	EphyBookmarkPropertiesPrivate *priv = properties->priv;
-	
-	if(priv->duplicate_idle != 0)
+	if (properties->duplicate_idle != 0)
 	{
-		g_source_remove (priv->duplicate_idle);
+		g_source_remove (properties->duplicate_idle);
 	}
-	
-	priv->duplicate_idle = g_timeout_add 
+
+	properties->duplicate_idle = g_timeout_add
 	  (500, (GSourceFunc)update_warning, properties);
-	g_source_set_name_by_id (priv->duplicate_idle, "[epiphany] update_warning");
+	g_source_set_name_by_id (properties->duplicate_idle, "[epiphany] update_warning");
 }
 
 static void
@@ -136,7 +144,7 @@ static void
 node_destroy_cb (EphyNode *bookmark,
 		 GtkWidget *dialog)
 {
-	EPHY_BOOKMARK_PROPERTIES (dialog)->priv->creating = FALSE;
+	EPHY_BOOKMARK_PROPERTIES (dialog)->creating = FALSE;
 	gtk_widget_destroy (dialog);
 }
 
@@ -144,58 +152,54 @@ static void
 ephy_bookmark_properties_set_bookmark (EphyBookmarkProperties *properties,
 				       EphyNode *bookmark)
 {
-	EphyBookmarkPropertiesPrivate *priv = properties->priv;
-
 	LOG ("Set bookmark");
-	
-	if (priv->bookmark)
+
+	if (properties->bookmark)
 	{
-		ephy_node_signal_disconnect_object (priv->bookmark,
+		ephy_node_signal_disconnect_object (properties->bookmark,
 						    EPHY_NODE_DESTROY,
 						    (EphyNodeCallback) node_destroy_cb,
 						    G_OBJECT (properties));
 	}
 
-	priv->bookmark = bookmark;
+	properties->bookmark = bookmark;
 
-	ephy_node_signal_connect_object (priv->bookmark,
+	ephy_node_signal_connect_object (properties->bookmark,
 					 EPHY_NODE_DESTROY,
 					 (EphyNodeCallback) node_destroy_cb,
 					 G_OBJECT (properties));
 }
 
 static void
-bookmark_properties_destroy_cb (GtkDialog *dialog,
-				gpointer data)
+ephy_bookmark_properties_destroy_cb (GtkDialog *dialog,
+                                     gpointer   data)
 {
 	EphyBookmarkProperties *properties = EPHY_BOOKMARK_PROPERTIES (dialog);
-	EphyBookmarkPropertiesPrivate *priv = properties->priv;
 
-	if (priv->creating)
+	if (properties->creating)
 	{
-		ephy_node_unref (priv->bookmark);
-		priv->creating = FALSE;
+		ephy_node_unref (properties->bookmark);
+		properties->creating = FALSE;
 	}
-	
-	if(priv->duplicate_idle != 0)
+
+	if (properties->duplicate_idle != 0)
 	{
-		g_source_remove (priv->duplicate_idle);
-		priv->duplicate_idle = 0;
+		g_source_remove (properties->duplicate_idle);
+		properties->duplicate_idle = 0;
 	}
 }
 
 static void
-bookmark_properties_response_cb (GtkDialog *dialog,
-				 int response_id,
-				 gpointer data)
+ephy_bookmark_properties_response_cb (GtkDialog *dialog,
+                                      int        response_id,
+                                      gpointer   data)
 {
 	EphyBookmarkProperties *properties = EPHY_BOOKMARK_PROPERTIES (dialog);
-	EphyBookmarkPropertiesPrivate *priv = properties->priv;
 
 	switch (response_id)
 	{
 	 	case GTK_RESPONSE_ACCEPT:
-			priv->creating = FALSE;
+			properties->creating = FALSE;
 			break;
 	 	default:
 			break;
@@ -205,7 +209,7 @@ bookmark_properties_response_cb (GtkDialog *dialog,
 }
 
 static void
-update_entry (EphyBookmarkProperties *props,
+update_entry (EphyBookmarkProperties *properties,
 	      GtkWidget *entry,
 	      guint prop)
 {
@@ -215,7 +219,7 @@ update_entry (EphyBookmarkProperties *props,
 	text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_take_string (&value, text);
-	ephy_node_set_property (props->priv->bookmark,
+	ephy_node_set_property (properties->bookmark,
 				prop,
 				&value);
 	g_value_unset (&value);
@@ -224,14 +228,13 @@ update_entry (EphyBookmarkProperties *props,
 static void
 update_window_title (EphyBookmarkProperties *properties)
 {
-	EphyBookmarkPropertiesPrivate *priv = properties->priv;
 	char *title;
 	const char *tmp;
 
-	tmp = ephy_node_get_property_string (priv->bookmark,
+	tmp = ephy_node_get_property_string (properties->bookmark,
 					     EPHY_NODE_BMK_PROP_TITLE);
 
-	if (priv->creating)
+	if (properties->creating)
 		title = g_strdup (_("Add Bookmark"));
 	else
 		title = g_strdup_printf (_("“%s” Properties"), tmp);
@@ -241,203 +244,154 @@ update_window_title (EphyBookmarkProperties *properties)
 }
 
 static void
-title_entry_changed_cb (GtkWidget *entry,
-			EphyBookmarkProperties *props)
+title_entry_changed_cb (GtkWidget              *entry,
+                        EphyBookmarkProperties *properties)
 {
-	update_entry (props, entry, EPHY_NODE_BMK_PROP_TITLE);
-	update_window_title (props);
+	update_entry (properties, entry, EPHY_NODE_BMK_PROP_TITLE);
+	update_window_title (properties);
+}
+
+static void
+on_topic_edited (GtkCellRendererText    *renderer,
+                 const char             *path_str,
+                 const char             *new_text,
+                 EphyBookmarkProperties *properties)
+{
+	if (*new_text != 0)
+	{
+		EphyNode *node;
+		node = ephy_bookmarks_add_keyword (properties->bookmarks, new_text);
+		ephy_bookmarks_set_keyword (properties->bookmarks, node, properties->bookmark);
+	}
+	else
+	{
+		ephy_topics_palette_update_list (properties->topics_model);
+	}
+}
+
+static void
+on_topic_toggled (GtkCellRendererToggle  *cell_renderer,
+                  gchar                  *path,
+                  EphyBookmarkProperties *properties)
+{
+	EphyNode     *topic;
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+
+	model = gtk_tree_view_get_model (properties->topics_tree_view);
+
+	g_return_if_fail (gtk_tree_model_get_iter_from_string (model, &iter, path));
+
+	gtk_tree_model_get (model, &iter, EPHY_TOPICS_PALETTE_COLUMN_NODE, &topic, -1);
+
+	/* Protect against toggling separators. */
+	if (topic == NULL)
+		return;
+
+	if (ephy_node_has_child (topic, properties->bookmark))
+	{
+		ephy_bookmarks_unset_keyword (properties->bookmarks,
+		                              topic,
+		                              properties->bookmark);
+	}
+	else
+	{
+		ephy_bookmarks_set_keyword (properties->bookmarks,
+		                            topic,
+		                            properties->bookmark);
+	}
+}
+
+static gboolean
+is_separator (GtkTreeModel *model,
+              GtkTreeIter  *iter,
+              gpointer      data)
+{
+	EphyNode *node;
+	gtk_tree_model_get (model, iter, EPHY_TOPICS_PALETTE_COLUMN_NODE, &node, -1);
+	return (node == NULL);
 }
 
 static void
 location_entry_changed_cb (GtkWidget *entry,
 			   EphyBookmarkProperties *properties)
 {
-	EphyBookmarkPropertiesPrivate *priv = properties->priv;
-
-	ephy_bookmarks_set_address (priv->bookmarks,
-				    priv->bookmark,
+	ephy_bookmarks_set_address (properties->bookmarks,
+				    properties->bookmark,
 				    gtk_entry_get_text (GTK_ENTRY (entry)));
 }
 
-static void
-list_mapped_cb (GtkWidget *widget,
-		 EphyBookmarkProperties *properties)
-{
-	GdkGeometry geometry;
-	
-	geometry.min_width = -1;
-	geometry.min_height = 230;
-	gtk_window_set_geometry_hints (GTK_WINDOW (properties),
-				       widget, &geometry,
-				       GDK_HINT_MIN_SIZE);
-}
-
-static void
-list_unmapped_cb (GtkWidget *widget,
-		 EphyBookmarkProperties *properties)
-{
-	GdkGeometry geometry;
-	
-	geometry.max_height = -1;
-	geometry.max_width = G_MAXINT;
-	gtk_window_set_geometry_hints (GTK_WINDOW (properties),	
-				       GTK_WIDGET (properties),
-				       &geometry, GDK_HINT_MAX_SIZE);
-}
-
 static void
 ephy_bookmark_properties_init (EphyBookmarkProperties *properties)
 {
-	properties->priv = EPHY_BOOKMARK_PROPERTIES_GET_PRIVATE (properties);
-	gtk_window_set_modal (GTK_WINDOW (properties), TRUE);
 }
 
 static GObject *
-ephy_bookmark_properties_constructor (GType type,
-				      guint n_construct_properties,
-				      GObjectConstructParam *construct_params)
+ephy_bookmark_properties_constructor (GType                  type,
+                                      guint                  n_construct_properties,
+                                      GObjectConstructParam *construct_params)
 {
-	GObject *object;
+	GObject                *object;
 	EphyBookmarkProperties *properties;
-	EphyBookmarkPropertiesPrivate *priv;
-	GtkWidget *widget, *grid, *label, *entry, *container;
-	GtkWidget *content_area;
-	GtkWindow *window;
-	GtkDialog *dialog;
-	gboolean lockdown;
-	const char *tmp;
-	char *unescaped_url;
+
+	gboolean      lockdown;
+	const char   *tmp;
+	char         *unescaped_url;
+	GtkWidget    *entry;
 
 	object = G_OBJECT_CLASS (ephy_bookmark_properties_parent_class)->constructor (type,
                                                                                       n_construct_properties,
                                                                                       construct_params);
-
-	widget = GTK_WIDGET (object);
-	window = GTK_WINDOW (object);
-	dialog = GTK_DIALOG (object);
 	properties = EPHY_BOOKMARK_PROPERTIES (object);
-	priv = properties->priv;
 
-	gtk_window_set_type_hint (window, GDK_WINDOW_TYPE_HINT_DIALOG);
+	gtk_widget_init_template (GTK_WIDGET (properties));
 
-	g_signal_connect (properties, "response",
-			  G_CALLBACK (bookmark_properties_response_cb), properties);
-
-	g_signal_connect (properties, "destroy",
-			  G_CALLBACK (bookmark_properties_destroy_cb), properties);
-
-	if (!priv->creating)
+	if (!properties->creating)
 	{
-		ephy_initial_state_add_window (widget,
-                                               "bookmark_properties",
-                                               290, 280, FALSE,
-                                               EPHY_INITIAL_STATE_WINDOW_SAVE_POSITION |
-                                               EPHY_INITIAL_STATE_WINDOW_SAVE_SIZE);
+		ephy_initial_state_add_window (GTK_WIDGET (properties),
+		                               "bookmark_properties",
+		                               290, 280, FALSE,
+		                               EPHY_INITIAL_STATE_WINDOW_SAVE_POSITION |
+		                               EPHY_INITIAL_STATE_WINDOW_SAVE_SIZE);
 	}
 	/* Lockdown */
 	lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-					   EPHY_PREFS_LOCKDOWN_BOOKMARK_EDITING);
+	                                   EPHY_PREFS_LOCKDOWN_BOOKMARK_EDITING);
 
 	update_window_title (properties);
-	content_area = gtk_dialog_get_content_area (dialog);
 
-	gtk_container_set_border_width (GTK_CONTAINER (properties), 5);
-	gtk_box_set_spacing (GTK_BOX (content_area), 2);
+	gtk_editable_set_editable (GTK_EDITABLE (properties->title_entry), !lockdown);
+	tmp = ephy_node_get_property_string (properties->bookmark, EPHY_NODE_BMK_PROP_TITLE);
+	gtk_entry_set_text (properties->title_entry, tmp);
 
-	grid = gtk_grid_new ();
-	gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
-	gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
-	gtk_container_set_border_width (GTK_CONTAINER (grid), 5);
-	gtk_widget_show (grid);
-
-	entry = gtk_entry_new ();
-	gtk_editable_set_editable (GTK_EDITABLE (entry), !lockdown);
-	tmp = ephy_node_get_property_string (properties->priv->bookmark,
-					     EPHY_NODE_BMK_PROP_TITLE);
-	gtk_entry_set_text (GTK_ENTRY (entry), tmp);
-	g_signal_connect (entry, "changed",
-			  G_CALLBACK (title_entry_changed_cb), properties);
-	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
-	gtk_widget_set_size_request (entry, 200, -1);
-	gtk_widget_show (entry);
-	label = gtk_label_new_with_mnemonic (_("_Title:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-	gtk_widget_show (label);
-	gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
-	gtk_grid_attach (GTK_GRID (grid), entry, 1, 0, 1, 1);
-	gtk_widget_set_hexpand (entry, TRUE);
-
-	entry = gtk_entry_new ();
-	gtk_editable_set_editable (GTK_EDITABLE (entry), !lockdown);
-	tmp = ephy_node_get_property_string (properties->priv->bookmark,
-					     EPHY_NODE_BMK_PROP_LOCATION);
-	unescaped_url = g_uri_unescape_string (tmp, NULL);
-	gtk_entry_set_text (GTK_ENTRY (entry), unescaped_url);
-	g_signal_connect (entry, "changed",
-			  G_CALLBACK (location_entry_changed_cb), properties);
-	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
-	gtk_widget_show (entry);
-	label = gtk_label_new_with_mnemonic (_("A_ddress:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-	gtk_widget_show (label);
-	gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
-	gtk_grid_attach (GTK_GRID (grid), entry, 1, 1, 1, 1);
-	gtk_widget_set_hexpand (entry, TRUE);
+	gtk_editable_set_editable (GTK_EDITABLE (properties->adress_entry), !lockdown);
+	tmp = ephy_node_get_property_string (properties->bookmark, EPHY_NODE_BMK_PROP_LOCATION);
+	unescaped_url = ephy_uri_safe_unescape (tmp);
+	gtk_entry_set_text (properties->adress_entry, unescaped_url);
 	g_free (unescaped_url);
 
-	entry = ephy_topics_entry_new (priv->bookmarks, priv->bookmark);
+	entry = ephy_topics_entry_new (properties->bookmarks, properties->bookmark);
 	gtk_editable_set_editable (GTK_EDITABLE (entry), !lockdown);
-	priv->entry = entry;
+	gtk_label_set_mnemonic_widget (properties->topics_label, entry);
 	gtk_widget_show (entry);
-	label = gtk_label_new_with_mnemonic(_("T_opics:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-	gtk_widget_show (label);
-	gtk_grid_attach (GTK_GRID (grid), label, 0, 2, 1, 1);
-	gtk_grid_attach (GTK_GRID (grid), entry, 1, 2, 1, 1);
+	gtk_grid_attach (properties->grid, entry, 1, 2, 1, 1);
 	gtk_widget_set_hexpand (entry, TRUE);
 
-	widget = ephy_topics_palette_new (priv->bookmarks, priv->bookmark);
-	container = g_object_new (GTK_TYPE_SCROLLED_WINDOW,
-				  "hadjustment", NULL,
-				  "vadjustment", NULL,
-				  "hscrollbar_policy", GTK_POLICY_AUTOMATIC,
-				  "vscrollbar_policy", GTK_POLICY_AUTOMATIC,
-				  "shadow_type", GTK_SHADOW_IN,
-				  NULL);
-	gtk_container_add (GTK_CONTAINER (container), widget);
-	gtk_widget_show (widget);
-	gtk_widget_set_sensitive (container, !lockdown);
-	gtk_widget_show (container);
-	g_signal_connect (container, "map", G_CALLBACK (list_mapped_cb), properties);
-	g_signal_connect (container, "unmap", G_CALLBACK (list_unmapped_cb), properties);
+	gtk_tree_view_set_row_separator_func (properties->topics_tree_view, is_separator, NULL, NULL);
 
-	widget = gtk_expander_new (_("Sho_w all topics"));
-	gtk_expander_set_use_underline (GTK_EXPANDER (widget), TRUE);
-	ephy_initial_state_add_expander (widget, "bookmark_properties_list", FALSE);
-	gtk_container_add (GTK_CONTAINER (widget), container);
-	gtk_widget_show (widget);
-	gtk_grid_attach (GTK_GRID (grid), widget, 1, 3, 1, 1);
-	gtk_widget_set_hexpand (widget, TRUE);
-	gtk_widget_set_vexpand (widget, TRUE);
+	properties->topics_model = ephy_topics_palette_new (properties->bookmarks, properties->bookmark);
+	gtk_tree_view_set_model (properties->topics_tree_view, GTK_TREE_MODEL (properties->topics_model));
 
-	priv->warning = gtk_label_new (NULL);
-	gtk_grid_attach (GTK_GRID (grid), priv->warning, 0, 4, 2, 1);
-	gtk_widget_show (priv->warning);
+	/* TODO bind; and the entry?! */
+	gtk_widget_set_sensitive (GTK_WIDGET (properties->topics_scrolled_window), !lockdown);
 
-	gtk_box_pack_start (GTK_BOX (content_area), grid, TRUE, TRUE, 0);
+	ephy_initial_state_add_expander (GTK_WIDGET (properties->topics_expander), "bookmark_properties_list", FALSE);
 
-	if (priv->creating)
+	if (properties->creating)
 	{
-		gtk_dialog_add_button (dialog,
-				       _("_Cancel"),
-				       GTK_RESPONSE_CANCEL);
-		gtk_dialog_add_button (dialog,
-				       _("_Add"),
-				       GTK_RESPONSE_ACCEPT);
-		gtk_dialog_set_default_response (dialog, GTK_RESPONSE_ACCEPT);
+		gtk_dialog_add_button (GTK_DIALOG (properties), _("_Cancel"), GTK_RESPONSE_CANCEL);
+		gtk_dialog_add_button (GTK_DIALOG (properties), _("_Add"), GTK_RESPONSE_ACCEPT);
+		gtk_dialog_set_default_response (GTK_DIALOG (properties), GTK_RESPONSE_ACCEPT);
 	}
 
 	update_warning (properties);
@@ -452,14 +406,13 @@ ephy_bookmark_properties_set_property (GObject *object,
 				       GParamSpec *pspec)
 {
 	EphyBookmarkProperties *properties = EPHY_BOOKMARK_PROPERTIES (object);
-	EphyBookmarkPropertiesPrivate *priv = properties->priv;
 	EphyNode *bookmarks;
 
 	switch (prop_id)
 	{
 		case PROP_BOOKMARKS:
-			priv->bookmarks = g_value_get_object (value);
-			bookmarks = ephy_bookmarks_get_bookmarks (priv->bookmarks);
+			properties->bookmarks = g_value_get_object (value);
+			bookmarks = ephy_bookmarks_get_bookmarks (properties->bookmarks);
 			ephy_node_signal_connect_object (bookmarks,
 							 EPHY_NODE_CHILD_ADDED,
 							 (EphyNodeCallback) node_added_cb,
@@ -478,7 +431,7 @@ ephy_bookmark_properties_set_property (GObject *object,
 				(properties, g_value_get_pointer (value));
 			break;
 		case PROP_CREATING:
-			priv->creating = g_value_get_boolean (value);
+			properties->creating = g_value_get_boolean (value);
 			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -509,6 +462,7 @@ static void
 ephy_bookmark_properties_class_init (EphyBookmarkPropertiesClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->constructor = ephy_bookmark_properties_constructor;
 	object_class->set_property = ephy_bookmark_properties_set_property;
@@ -537,7 +491,25 @@ ephy_bookmark_properties_class_init (EphyBookmarkPropertiesClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-	g_type_class_add_private (object_class, sizeof (EphyBookmarkPropertiesPrivate));
+	/* from UI file */
+	gtk_widget_class_set_template_from_resource (widget_class,
+	                                             "/org/gnome/epiphany/bookmark-properties.ui");
+
+	gtk_widget_class_bind_template_child (widget_class, EphyBookmarkProperties, grid);
+	gtk_widget_class_bind_template_child (widget_class, EphyBookmarkProperties, title_entry);
+	gtk_widget_class_bind_template_child (widget_class, EphyBookmarkProperties, adress_entry);
+	gtk_widget_class_bind_template_child (widget_class, EphyBookmarkProperties, topics_label);
+	gtk_widget_class_bind_template_child (widget_class, EphyBookmarkProperties, topics_expander);
+	gtk_widget_class_bind_template_child (widget_class, EphyBookmarkProperties, topics_tree_view);
+	gtk_widget_class_bind_template_child (widget_class, EphyBookmarkProperties, topics_scrolled_window);
+	gtk_widget_class_bind_template_child (widget_class, EphyBookmarkProperties, warning_label);
+
+	gtk_widget_class_bind_template_callback (widget_class, title_entry_changed_cb);
+	gtk_widget_class_bind_template_callback (widget_class, on_topic_toggled);   // TODO make the row activatable instead of a little togglebutton
+	gtk_widget_class_bind_template_callback (widget_class, on_topic_edited);    // TODO topics’ names are not editable from there, they are synced with bookmarks... but this is used for... updating the entry \o/ yay \o/
+	gtk_widget_class_bind_template_callback (widget_class, location_entry_changed_cb);
+	gtk_widget_class_bind_template_callback (widget_class, ephy_bookmark_properties_response_cb);
+	gtk_widget_class_bind_template_callback (widget_class, ephy_bookmark_properties_destroy_cb);
 }
 
 /* public API */
@@ -560,5 +532,5 @@ ephy_bookmark_properties_new (EphyBookmarks *bookmarks,
 EphyNode *
 ephy_bookmark_properties_get_node (EphyBookmarkProperties *properties)
 {
-	return properties->priv->bookmark;
+	return properties->bookmark;
 }
