@@ -12,9 +12,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -37,6 +35,7 @@
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-initial-state.h"
+#include "ephy-uri-helpers.h"
 #include "ephy-topic-action.h"
 #include "ephy-window.h"
 #include "popup-commands.h"
@@ -897,7 +896,7 @@ cmd_bookmark_properties (GtkAction *action,
 	{
 		EphyNode *node = l->data;
 
-		ephy_bookmarks_ui_show_bookmark (node);
+		ephy_bookmarks_ui_show_bookmark (GTK_WINDOW (editor), node);
 	}
 
 	g_list_free (selection);
@@ -1484,7 +1483,7 @@ unescape_bookmark_uri (EphyNode *node,
 		       gpointer user_data)
 {
 	const char *url = g_value_get_string (value);
-	g_value_take_string (value, g_uri_unescape_string (url, NULL));
+	g_value_take_string (value, ephy_uri_safe_unescape (url));
 }
 
 
