@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -74,7 +72,7 @@ write_topics_list (EphyNode *topics,
 	for (i = 0; i < children->len; i++)
 	{
 		EphyNode *kid;
-		EphyNodePriority priority;
+		int priority;
 
 		kid = g_ptr_array_index (children, i);
 
