@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -32,6 +30,7 @@
 #include "ephy-history-service.h"
 #include "ephy-node-common.h"
 #include "ephy-prefs.h"
+#include "ephy-profile-utils.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-signal-accumulator.h"
@@ -220,6 +219,7 @@ ephy_bookmarks_save_delayed (EphyBookmarks *bookmarks, int delay)
 				g_timeout_add_seconds (BOOKMARKS_SAVE_DELAY,
 					       (GSourceFunc) save_bookmarks_delayed,
 					       bookmarks);
+			g_source_set_name_by_id (bookmarks->priv->save_timeout_id, "[epiphany] save_bookmarks_delayed");
 		}
 		else
 		{
@@ -230,34 +230,24 @@ ephy_bookmarks_save_delayed (EphyBookmarks *bookmarks, int delay)
 	}
 }
 
-#ifdef HAVE_WEBKIT2
-/* TODO: Favicons */
-#else
 static void
 icon_updated_cb (WebKitFaviconDatabase *favicon_database,
 		 const char *address,
+		 const char *icon,
 		 EphyBookmarks *eb)
 {
-	char *icon;
-
-	icon = webkit_favicon_database_get_favicon_uri (favicon_database, address);
 	ephy_bookmarks_set_icon (eb, address, icon);
-	g_free (icon);
 }
-#endif
 
 static void
 ephy_setup_history_notifiers (EphyBookmarks *eb)
 {
-#ifdef HAVE_WEBKIT2
-	/* TODO: Favicons */
-#else
 	WebKitFaviconDatabase *favicon_database;
-	
-	favicon_database = webkit_get_favicon_database ();
-	g_signal_connect (favicon_database, "icon-loaded",
+	EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+
+	favicon_database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (shell));
+	g_signal_connect (favicon_database, "favicon-changed",
 			  G_CALLBACK (icon_updated_cb), eb);
-#endif
 }
 
 static void
@@ -948,10 +938,10 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	eb->priv->db = db;
 
 	eb->priv->xml_file = g_build_filename (ephy_dot_dir (),
-					       "ephy-bookmarks.xml",
+					       EPHY_BOOKMARKS_FILE,
 					       NULL);
 	eb->priv->rdf_file = g_build_filename (ephy_dot_dir (),
-					       "bookmarks.rdf",
+					       EPHY_BOOKMARKS_FILE_RDF,
 					       NULL);
 
 	/* Bookmarks */
@@ -1143,11 +1133,8 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 		    const char *url)
 {
 	EphyNode *bm;
-#ifdef HAVE_WEBKIT2
-	/* TODO: Favicons */
-#else
 	WebKitFaviconDatabase *favicon_database;
-#endif
+	EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
 	bm = ephy_node_new (eb->priv->db);
 
@@ -1162,10 +1149,7 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 	}
 	ephy_node_set_property_string (bm, EPHY_NODE_BMK_PROP_TITLE, title);
 
-#ifdef HAVE_WEBKIT2
-	/* TODO: Favicons */
-#else
-	favicon_database = webkit_get_favicon_database ();
+	favicon_database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (shell));
 	if (favicon_database != NULL)
 	{
 		char *icon = webkit_favicon_database_get_favicon_uri (favicon_database, url);
@@ -1176,7 +1160,6 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 			g_free (icon);
 		}
 	}
-#endif
 
 	update_has_smart_address (eb, bm, url);
 	update_bookmark_keywords (eb, bm);
