@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -26,13 +24,15 @@
 #include "ephy-favicon-helpers.h"
 #include "ephy-history-service.h"
 #include "ephy-shell.h"
+#include "ephy-uri-helpers.h"
 
 #include <string.h>
 
 enum {
   PROP_0,
   PROP_HISTORY_SERVICE,
-  PROP_BOOKMARKS
+  PROP_BOOKMARKS,
+  PROP_USE_MARKUP
 };
 
 G_DEFINE_TYPE (EphyCompletionModel, ephy_completion_model, GTK_TYPE_LIST_STORE)
@@ -46,6 +46,8 @@ struct _EphyCompletionModelPrivate {
   EphyNode *bookmarks;
   EphyNode *smart_bookmarks;
   GSList *search_terms;
+
+  gboolean use_markup;
 };
 
 static void
@@ -85,8 +87,11 @@ ephy_completion_model_set_property (GObject *object, guint property_id, const GV
 
     self->priv->bookmarks = ephy_bookmarks_get_bookmarks (bookmarks);
     self->priv->smart_bookmarks = ephy_bookmarks_get_smart_bookmarks (bookmarks);
+    }
+    break;
+  case PROP_USE_MARKUP:
+    self->priv->use_markup = g_value_get_boolean (value);
     break;
-  }
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
     break;
@@ -133,6 +138,14 @@ ephy_completion_model_class_init (EphyCompletionModelClass *klass)
                                                          "The bookmarks",
                                                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (object_class,
+                                   PROP_USE_MARKUP,
+                                   g_param_spec_boolean ("use-markup",
+                                                         "Whether we should be using markup",
+                                                         "Whether we should be using markup",
+                                                         TRUE,
+                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
   g_type_class_add_private (object_class, sizeof (EphyCompletionModelPrivate));
 }
 
@@ -247,7 +260,7 @@ get_row_text (const gchar *url, const gchar *title, const gchar *subtitle_color)
   if (!url)
     return g_markup_escape_text (title, -1);
 
-  unescaped_url = g_uri_unescape_string (url, NULL);
+  unescaped_url = ephy_uri_safe_unescape (url);
   if (g_strcmp0 (url, title) == 0)
     text = g_markup_escape_text (unescaped_url, -1);
   else
@@ -269,7 +282,11 @@ set_row_in_model (EphyCompletionModel *model, int position, PotentialRow *row, c
 
   database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (shell));
 
-  text = get_row_text (row->location, row->title, subtitle_color);
+  if (model->priv->use_markup)
+    text = get_row_text (row->location, row->title, subtitle_color);
+  else
+    text = g_strdup (row->title);
+
   gtk_list_store_insert_with_values (GTK_LIST_STORE (model), &iter, position,
                                      EPHY_COMPLETION_TEXT_COL, text ? text : "",
                                      EPHY_COMPLETION_URL_COL, row->location,
@@ -321,14 +338,15 @@ replace_rows_in_model (EphyCompletionModel *model, GSList *new_rows)
   /* This is by far the simplest way of doing, and yet it gives
    * basically the same result than the other methods... */
   int i;
-  gchar *subtitle_color;
+  gchar *subtitle_color = NULL;
 
   gtk_list_store_clear (GTK_LIST_STORE (model));
 
   if (!new_rows)
     return;
 
-  subtitle_color = get_text_column_subtitle_color ();
+  if (model->priv->use_markup)
+    subtitle_color = get_text_column_subtitle_color ();
 
   for (i = 0; new_rows != NULL; i++) {
     PotentialRow *row = (PotentialRow*)new_rows->data;
@@ -650,7 +668,8 @@ ephy_completion_model_update_for_string (EphyCompletionModel *model,
 
 EphyCompletionModel *
 ephy_completion_model_new (EphyHistoryService *history_service,
-                           EphyBookmarks      *bookmarks)
+                           EphyBookmarks      *bookmarks,
+                           gboolean            use_markup)
 {
   g_return_val_if_fail (EPHY_IS_HISTORY_SERVICE (history_service), NULL);
   g_return_val_if_fail (EPHY_IS_BOOKMARKS (bookmarks), NULL);
@@ -658,5 +677,6 @@ ephy_completion_model_new (EphyHistoryService *history_service,
   return g_object_new (EPHY_TYPE_COMPLETION_MODEL,
                        "history-service", history_service,
                        "bookmarks", bookmarks,
+                       "use-markup", use_markup,
                        NULL);
 }
