@@ -15,9 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -48,20 +46,16 @@ struct _EphyHistoryWindowPrivate
 	GCancellable *cancellable;
 
 	GtkWidget *treeview;
+	GtkTreeSelection *tree_selection;
 	GtkWidget *liststore;
 	GtkTreeViewColumn *date_column;
 	GtkTreeViewColumn *name_column;
 	GtkTreeViewColumn *location_column;
 	GtkWidget *date_renderer;
 	GtkWidget *location_renderer;
-	GtkWidget *remove_button;
-	GtkWidget *open_button;
-	GtkWidget *clear_button;
-	GtkWidget *open_menuitem;
-	GtkWidget *bookmark_menuitem;
-	GtkWidget *delete_menuitem;
-	GtkWidget *copy_location_menuitem;
-	GtkWidget *treeview_popup_menu;
+	GMenuModel *treeview_popup_menu_model;
+
+	GActionGroup *action_group;
 
 	GList *urls;
 	guint sorter_source;
@@ -94,7 +88,6 @@ typedef enum
 static gboolean
 add_urls_source (EphyHistoryWindow *self)
 {
-	GtkListStore *store = GTK_LIST_STORE (self->priv->liststore);
 	EphyHistoryURL *url;
 	GTimer *timer;
 	GList *element;
@@ -111,7 +104,7 @@ add_urls_source (EphyHistoryWindow *self)
 	do {
 		element = self->priv->urls;
 		url = element->data;
-		gtk_list_store_insert_with_values (store,
+		gtk_list_store_insert_with_values (GTK_LIST_STORE (self->priv->liststore),
 						   NULL, G_MAXINT,
 						   COLUMN_DATE, url->last_visit_time,
 						   COLUMN_NAME, url->title,
@@ -217,7 +210,7 @@ filter_now (EphyHistoryWindow *self)
 	}
 
 	remove_pending_sorter_source (self);
-	
+
 	ephy_history_service_find_urls (self->priv->history_service,
 					from, to,
 					NUM_RESULTS_LIMIT, 0,
@@ -261,11 +254,11 @@ confirmation_dialog_construct (EphyHistoryWindow *self)
 
 	gtk_window_group_add_window (ephy_gui_ensure_window_group (GTK_WINDOW (self)),
 				     GTK_WINDOW (dialog));
-	
+
 	button = gtk_button_new_with_mnemonic (_("Cl_ear"));
 	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_ACCEPT);
-	
+
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
 	g_signal_connect (dialog, "response",
@@ -276,8 +269,12 @@ confirmation_dialog_construct (EphyHistoryWindow *self)
 }
 
 static void
-clear_all_history (EphyHistoryWindow *self)
+forget_all (GSimpleAction     *action,
+            GVariant          *parameter,
+            gpointer           user_data)
 {
+	EphyHistoryWindow *self = EPHY_HISTORY_WINDOW (user_data);
+
 	if (self->priv->confirmation_dialog == NULL)
 	{
 		GtkWidget **confirmation_dialog;
@@ -350,11 +347,9 @@ get_selection_foreach (GtkTreeModel *model,
 static GList *
 get_selection (EphyHistoryWindow *self)
 {
-	GtkTreeSelection *selection;
 	GList *list = NULL;
 
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (self->priv->treeview));
-	gtk_tree_selection_selected_foreach (selection,
+	gtk_tree_selection_selected_foreach (self->priv->tree_selection,
 					     (GtkTreeSelectionForeachFunc) get_selection_foreach,
 					     &list);
 
@@ -372,11 +367,14 @@ delete_selected (EphyHistoryWindow *self)
 }
 
 static void
-open_selected (EphyHistoryWindow *self)
+open_selection (GSimpleAction     *action,
+                GVariant          *parameter,
+                gpointer           user_data)
 {
+	EphyHistoryWindow *self = EPHY_HISTORY_WINDOW (user_data);
+	EphyWindow *window;
 	GList *selection;
 	GList *l;
-	EphyWindow *window;
 
 	selection = get_selection (self);
 
@@ -394,16 +392,11 @@ open_selected (EphyHistoryWindow *self)
 }
 
 static void
-on_open_menuitem_activate (GtkMenuItem *menuitem,
-			   EphyHistoryWindow *self)
-{
-	open_selected (self);
-}
-
-static void
-on_copy_location_menuitem_activate (GtkMenuItem *menuitem,
-				    EphyHistoryWindow *self)
+copy_url (GSimpleAction     *action,
+          GVariant          *parameter,
+          gpointer           user_data)
 {
+	EphyHistoryWindow *self = EPHY_HISTORY_WINDOW (user_data);
 	GList *selection;
 
 	selection = get_selection (self);
@@ -418,9 +411,11 @@ on_copy_location_menuitem_activate (GtkMenuItem *menuitem,
 }
 
 static void
-on_bookmark_menuitem_activate (GtkMenuItem *menuitem,
-			       EphyHistoryWindow *self)
+bookmark (GSimpleAction     *action,
+          GVariant          *parameter,
+          gpointer           user_data)
 {
+	EphyHistoryWindow *self = EPHY_HISTORY_WINDOW (user_data);
 	GList *selection;
 
 	selection = get_selection (self);
@@ -438,9 +433,12 @@ on_bookmark_menuitem_activate (GtkMenuItem *menuitem,
 }
 
 static void
-on_delete_menuitem_activate (GtkMenuItem *menuitem,
-			     EphyHistoryWindow *self)
+forget (GSimpleAction     *action,
+        GVariant          *parameter,
+        gpointer           user_data)
 {
+	EphyHistoryWindow *self = EPHY_HISTORY_WINDOW (user_data);
+
 	delete_selected (self);
 }
 
@@ -459,32 +457,41 @@ on_treeview_key_press_event (GtkWidget         *widget,
 	return FALSE;
 }
 
+static void
+update_popup_menu_actions (GActionGroup *action_group,
+                           gboolean      only_one_selected_item)
+{
+	GAction *copy_url_action;
+	GAction *bookmark_action;
+	gboolean bookmarks_locked;
+
+	copy_url_action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "copy-url");
+	bookmark_action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "bookmark");
+	bookmarks_locked = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+	                                           EPHY_PREFS_LOCKDOWN_BOOKMARK_EDITING);
+
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (copy_url_action), only_one_selected_item);
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (bookmark_action), (only_one_selected_item && !bookmarks_locked));
+}
+
 static gboolean
 on_treeview_button_press_event (GtkWidget         *widget,
 				GdkEventButton    *event,
 				EphyHistoryWindow *self)
 {
 	if (event->button == 3) {
-		GtkTreeSelection *selection;
 		int n;
-		gboolean bookmarks_locked;
+		GtkWidget *menu;
 
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (self->priv->treeview));
-		n = gtk_tree_selection_count_selected_rows (selection);
-		if (n == 0)
+		n = gtk_tree_selection_count_selected_rows (self->priv->tree_selection);
+		if (n <= 0)
 			return FALSE;
 
-		gtk_widget_set_sensitive (self->priv->open_menuitem, (n > 0));
-		gtk_widget_set_sensitive (self->priv->copy_location_menuitem, (n > 0));
-		gtk_widget_set_sensitive (self->priv->delete_menuitem, (n > 0));
+		update_popup_menu_actions (self->priv->action_group, (n == 1));
 
-		bookmarks_locked = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-							   EPHY_PREFS_LOCKDOWN_BOOKMARK_EDITING);
-		gtk_widget_set_sensitive (self->priv->bookmark_menuitem, (n == 1 && !bookmarks_locked));
-
-		gtk_menu_popup (GTK_MENU (self->priv->treeview_popup_menu),
-				NULL, NULL, NULL, NULL,
-				event->button, event->time);
+		menu = gtk_menu_new_from_model (self->priv->treeview_popup_menu_model);
+		gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (self), NULL);
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
 		return TRUE;
 	}
 
@@ -525,16 +532,26 @@ on_search_entry_changed (GtkSearchEntry *entry,
 	filter_now (self);
 }
 
+static void
+update_selection_actions (GActionGroup *action_group,
+                          gboolean      has_selection)
+{
+	GAction *forget_action;
+	GAction *open_selection_action;
+
+	forget_action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "forget");
+	open_selection_action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "open-selection");
+
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (forget_action), has_selection);
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (open_selection_action), has_selection);
+}
+
 static void
 on_treeview_selection_changed (GtkTreeSelection *selection,
 			       EphyHistoryWindow *self)
 {
-	gboolean has_selection;
-
-	has_selection = gtk_tree_selection_count_selected_rows (selection) > 0;
-
-	gtk_widget_set_sensitive (self->priv->remove_button, has_selection);
-	gtk_widget_set_sensitive (self->priv->open_button, has_selection);
+	update_selection_actions (self->priv->action_group,
+	                          gtk_tree_selection_count_selected_rows (selection) > 0);
 }
 
 static void
@@ -562,23 +579,6 @@ on_treeview_column_clicked_event (GtkTreeViewColumn *column,
 	filter_now (self);
 }
 
-static void
-on_remove_button_clicked (GtkButton *button,
-			  EphyHistoryWindow *self)
-{
-	delete_selected (self);
-
-	/* Restore the focus to the button */
-	gtk_widget_grab_focus (GTK_WIDGET (button));
-}
-
-static void
-on_open_button_clicked (GtkButton *button,
-			EphyHistoryWindow *self)
-{
-	open_selected (self);
-}
-
 static gboolean
 on_urls_visited_cb (EphyHistoryService *service,
 		    EphyHistoryWindow *self)
@@ -715,33 +715,20 @@ ephy_history_window_class_init (EphyHistoryWindowClass *klass)
 	                                             "/org/gnome/epiphany/history-dialog.ui");
 	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, liststore);
 	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, treeview);
-	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, clear_button);
-	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, remove_button);
-	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, open_button);
+	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, tree_selection);
 	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, date_column);
 	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, name_column);
 	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, location_column);
 	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, date_renderer);
 	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, location_renderer);
-	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, open_menuitem);
-	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, copy_location_menuitem);
-	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, bookmark_menuitem);
-	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, delete_menuitem);
-	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, treeview_popup_menu);
+	gtk_widget_class_bind_template_child_private (widget_class, EphyHistoryWindow, treeview_popup_menu_model);
 
 	gtk_widget_class_bind_template_callback (widget_class, on_treeview_row_activated);
 	gtk_widget_class_bind_template_callback (widget_class, on_treeview_key_press_event);
 	gtk_widget_class_bind_template_callback (widget_class, on_treeview_button_press_event);
 	gtk_widget_class_bind_template_callback (widget_class, on_treeview_selection_changed);
 	gtk_widget_class_bind_template_callback (widget_class, on_treeview_column_clicked_event);
-	gtk_widget_class_bind_template_callback (widget_class, on_remove_button_clicked);
-	gtk_widget_class_bind_template_callback (widget_class, on_open_button_clicked);
 	gtk_widget_class_bind_template_callback (widget_class, on_search_entry_changed);
-
-	gtk_widget_class_bind_template_callback (widget_class, on_open_menuitem_activate);
-	gtk_widget_class_bind_template_callback (widget_class, on_copy_location_menuitem_activate);
-	gtk_widget_class_bind_template_callback (widget_class, on_bookmark_menuitem_activate);
-	gtk_widget_class_bind_template_callback (widget_class, on_delete_menuitem_activate);
 }
 
 static void
@@ -790,41 +777,6 @@ convert_location_data_func (GtkTreeViewColumn *column,
 	g_free (unescaped_url);
 }
 
-static void
-response_cb (GtkDialog *widget,
-	     int response,
-	     EphyHistoryWindow *self)
-{
-	if (response == GTK_RESPONSE_REJECT) {
-		clear_all_history (self);
-		return;
-	}
-
-	gtk_widget_destroy (GTK_WIDGET (self));
-}
-
-void
-ephy_history_window_set_parent (EphyHistoryWindow *ebe,
-				GtkWidget *window)
-{
-	GtkWidget **widget;
-	if (ebe->priv->window)
-	{
-		widget = &ebe->priv->window;
-		g_object_remove_weak_pointer
-			(G_OBJECT(ebe->priv->window),
-			 (gpointer *)widget);
-	}
-
-	ebe->priv->window = window;
-	widget = &ebe->priv->window;
-
-	g_object_add_weak_pointer
-			(G_OBJECT(ebe->priv->window),
-			(gpointer *)widget);
-
-}
-
 GtkWidget *
 ephy_history_window_new (EphyHistoryService *history_service)
 {
@@ -840,6 +792,24 @@ ephy_history_window_new (EphyHistoryService *history_service)
 	return GTK_WIDGET (self);
 }
 
+static GActionGroup *
+create_action_group (EphyHistoryWindow *self)
+{
+	const GActionEntry entries[] = {
+		{ "open-selection", open_selection },
+		{ "copy-url",       copy_url },
+		{ "bookmark",       bookmark },
+		{ "forget",         forget },
+		{ "forget-all",     forget_all }
+	};
+	GSimpleActionGroup *group;
+
+	group = g_simple_action_group_new ();
+	g_action_map_add_action_entries (G_ACTION_MAP (group), entries, G_N_ELEMENTS (entries), self);
+
+	return G_ACTION_GROUP (group);
+}
+
 static void
 ephy_history_window_init (EphyHistoryWindow *self)
 {
@@ -874,6 +844,8 @@ ephy_history_window_init (EphyHistoryWindow *self)
 						 GINT_TO_POINTER (COLUMN_LOCATION),
 						 NULL);
 
-	g_signal_connect (self, "response",
-			  G_CALLBACK (response_cb), self);
+	self->priv->action_group = create_action_group (self);
+	gtk_widget_insert_action_group (GTK_WIDGET (self), "history", self->priv->action_group);
+
+	update_selection_actions (self->priv->action_group, FALSE);
 }
