@@ -15,9 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -71,7 +69,7 @@ arbitrary_url_cb (GSettings *settings,
     if (embed == NULL)
       return;
 
-    address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
+    address = ephy_web_view_get_display_address (ephy_embed_get_web_view (embed));
     ephy_window_set_location (window, address);
     ephy_web_view_set_typed_address (ephy_embed_get_web_view (embed), NULL);
   }
