@@ -15,9 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -509,7 +507,11 @@ sync_icon (EphyWebView *view,
 static void
 sync_label (EphyEmbed *embed, GParamSpec *pspec, GtkWidget *label)
 {
-	gtk_label_set_text (GTK_LABEL (label), ephy_embed_get_title (embed));
+	const char *title;
+
+	title = ephy_embed_get_title (embed);
+	gtk_label_set_text (GTK_LABEL (label), title);
+	gtk_widget_set_tooltip_text (label, title);
 }
 
 static void
@@ -543,7 +545,7 @@ tab_label_style_set_cb (GtkWidget *hbox,
 
 	context = gtk_widget_get_pango_context (hbox);
 	style = gtk_widget_get_style_context (hbox);
-	gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL,
+	gtk_style_context_get (style, gtk_style_context_get_state (style),
 			       "font", &font_desc, NULL);
 	metrics = pango_context_get_metrics (context,
 					     font_desc,
