@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
@@ -68,8 +66,8 @@ int		ephy_notebook_add_tab		(EphyNotebook *nb,
 						 int position,
 						 gboolean jump_to);
 	
-void		ephy_notebook_set_show_tabs	(EphyNotebook *nb,
-						 gboolean show_tabs);
+void		ephy_notebook_set_tabs_allowed	(EphyNotebook *nb,
+						 gboolean tabs_allowed);
 
 void            ephy_notebook_next_page         (EphyNotebook *notebook);
 
