@@ -11,10 +11,8 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with the Control Center; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -22,7 +20,6 @@
 #include "ephy-search-provider.h"
 
 #include "ephy-completion-model.h"
-#include "ephy-embed-shell.h"
 #include "ephy-file-helpers.h"
 #include "ephy-prefs.h"
 #include "ephy-profile-utils.h"
@@ -41,6 +38,7 @@ struct _EphySearchProvider
   GCancellable             *cancellable;
 
   GSettings                *settings;
+  EphyHistoryService       *history_service;
   EphyBookmarks            *bookmarks;
   EphyCompletionModel      *model;
 };
@@ -108,7 +106,6 @@ gather_results_async (EphySearchProvider   *self,
   char *search_string;
 
   task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_check_cancellable (task, TRUE);
 
   search_string = g_strjoinv (" ", terms);
   g_task_set_task_data (task, search_string, g_free);
@@ -291,8 +288,13 @@ launch_search (EphySearchProvider  *self,
 
   search_string = g_strjoinv (" ", terms);
   query_param = soup_form_encode ("q", search_string, NULL);
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+  /* Format string under control of user input... but gsettings is trusted input. */
   /* + 2 here is getting rid of 'q=' */
   effective_url = g_strdup_printf (url_search, query_param + 2);
+#pragma GCC diagnostic pop
 
   launch_uri (effective_url, timestamp);
 
@@ -346,7 +348,6 @@ static void
 ephy_search_provider_init (EphySearchProvider *self)
 {
   char *filename;
-  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
   g_application_set_flags (G_APPLICATION (self), G_APPLICATION_IS_SERVICE);
 
@@ -366,10 +367,9 @@ ephy_search_provider_init (EphySearchProvider *self)
   self->settings = g_settings_new (EPHY_PREFS_SCHEMA);
 
   filename = g_build_filename (ephy_dot_dir (), EPHY_HISTORY_FILE, NULL);
+  self->history_service = ephy_history_service_new (filename, TRUE);
   self->bookmarks = ephy_bookmarks_new ();
-  self->model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (shell)),
-                                           self->bookmarks,
-                                           FALSE);
+  self->model = ephy_completion_model_new (self->history_service, self->bookmarks, FALSE);
   g_free (filename);
 
   self->cancellable = g_cancellable_new ();
@@ -428,6 +428,7 @@ ephy_search_provider_dispose (GObject *object)
   g_clear_object (&self->settings);
   g_clear_object (&self->cancellable);
   g_clear_object (&self->model);
+  g_clear_object (&self->history_service);
   g_clear_object (&self->bookmarks);
 
   G_OBJECT_CLASS (ephy_search_provider_parent_class)->dispose (object);
