@@ -15,9 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -26,7 +24,7 @@
 #include "ephy-bookmarks-editor.h"
 #include "ephy-bookmarks-import.h"
 #include "ephy-debug.h"
-#include "ephy-download.h"
+#include "ephy-downloads-manager.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
@@ -67,7 +65,7 @@ struct _EphyShellPrivate {
   GSList *open_uris_idle_ids;
 };
 
-EphyShell *ephy_shell = NULL;
+static EphyShell *ephy_shell = NULL;
 
 static void ephy_shell_class_init (EphyShellClass *klass);
 static void ephy_shell_init   (EphyShell *shell);
@@ -255,15 +253,51 @@ static GActionEntry app_normal_mode_entries[] = {
   { "reopen-closed-tab", reopen_closed_tab, NULL, NULL, NULL },
 };
 
+static void
+download_started_cb (WebKitWebContext *web_context,
+                     WebKitDownload *download,
+                     EphyShell *shell)
+{
+  EphyDownload *ephy_download;
+  gboolean ephy_download_set;
+
+  /* Is download locked down? */
+  if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+                              EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK)) {
+    webkit_download_cancel (download);
+    return;
+  }
+
+  /* Only create an EphyDownload for the WebKitDownload if it doesn't exist yet.
+   * This can happen when the download has been started automatically by WebKit,
+   * due to a context menu action or policy checker decision. Downloads started
+   * explicitly by Epiphany are marked with ephy-download-set GObject data.
+   */
+  ephy_download_set = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (download), "ephy-download-set"));
+  if (ephy_download_set)
+    return;
+
+  ephy_download = ephy_download_new (download);
+  ephy_downloads_manager_add_download (ephy_embed_shell_get_downloads_manager (EPHY_EMBED_SHELL (shell)),
+                                       ephy_download);
+  g_object_unref (ephy_download);
+}
+
 static void
 ephy_shell_startup (GApplication* application)
 {
+  EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (application);
   EphyEmbedShellMode mode;
 
   G_APPLICATION_CLASS (ephy_shell_parent_class)->startup (application);
 
   /* We're not remoting; start our services */
-  mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (application));
+  g_signal_connect (ephy_embed_shell_get_web_context (embed_shell),
+                    "download-started",
+                    G_CALLBACK (download_started_cb),
+                    application);
+
+  mode = ephy_embed_shell_get_mode (embed_shell);
 
   if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION) {
     GtkBuilder *builder;
@@ -319,9 +353,12 @@ ephy_shell_activate (GApplication *application)
    * can be invalidated by another remote instance.
    */
   if (priv->remote_startup_context == NULL) {
-    ephy_session_resume (ephy_shell_get_session (shell),
-                         priv->local_startup_context->user_time,
-                         NULL, session_load_cb, priv->local_startup_context);
+    if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell)) != EPHY_EMBED_SHELL_MODE_APPLICATION) {
+      ephy_session_resume (ephy_shell_get_session (shell),
+                           priv->local_startup_context->user_time,
+                           NULL, session_load_cb, priv->local_startup_context);
+    } else
+      ephy_shell_startup_continue (shell, priv->local_startup_context);
   } else {
     ephy_shell_startup_continue (shell, priv->remote_startup_context);
     g_clear_pointer (&priv->remote_startup_context, ephy_shell_startup_context_free);
@@ -515,56 +552,10 @@ ephy_shell_class_init (EphyShellClass *klass)
   g_type_class_add_private (object_class, sizeof(EphyShellPrivate));
 }
 
-static void
-download_started_cb (WebKitWebContext *web_context,
-                     WebKitDownload *download,
-                     EphyShell *shell)
-{
-  GtkWindow *window = NULL;
-  WebKitWebView *web_view;
-  EphyDownload *ephy_download;
-  gboolean ephy_download_set;
-
-  /* Is download locked down? */
-  if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-                              EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK)) {
-    webkit_download_cancel (download);
-    return;
-  }
-
-  /* Only create an EphyDownload for the WebKitDownload if it doesn't exist yet.
-   * This can happen when the download has been started automatically by WebKit,
-   * due to a context menu action or policy checker decision. Downloads started
-   * explicitly by Epiphany are marked with ephy-download-set GObject data.
-   */
-  ephy_download_set = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (download), "ephy-download-set"));
-  if (ephy_download_set)
-    return;
-
-  web_view = webkit_download_get_web_view (download);
-  if (web_view) {
-    GtkWidget *toplevel;
-
-    toplevel = gtk_widget_get_toplevel (GTK_WIDGET (web_view));
-    if (GTK_IS_WINDOW (toplevel))
-      window = GTK_WINDOW (toplevel);
-  }
-
-  if (!window)
-    window = gtk_application_get_active_window (GTK_APPLICATION (shell));
-
-  ephy_download = ephy_download_new (download, window);
-  ephy_window_add_download (EPHY_WINDOW (window), ephy_download);
-  g_object_unref (ephy_download);
-}
-
 static void
 ephy_shell_init (EphyShell *shell)
 {
   EphyShell **ptr = &ephy_shell;
-  WebKitWebContext *web_context;
-  EphyEmbedShellMode mode;
-  char *favicon_db_path;
 
   shell->priv = EPHY_SHELL_GET_PRIVATE (shell);
 
@@ -573,24 +564,6 @@ ephy_shell_init (EphyShell *shell)
   ephy_shell = shell;
   g_object_add_weak_pointer (G_OBJECT (ephy_shell),
                              (gpointer *)ptr);
-
-  web_context = webkit_web_context_get_default ();
-  g_signal_connect (web_context, "download-started",
-                    G_CALLBACK (download_started_cb),
-                    shell);
-
-  /* Do not ignore TLS errors. */
-  webkit_web_context_set_tls_errors_policy (web_context, WEBKIT_TLS_ERRORS_POLICY_FAIL);
-
-  /* Initialize the favicon cache as early as possible, or further
-     calls to webkit_web_context_get_favicon_database will fail. */
-  mode = ephy_embed_shell_get_mode (ephy_embed_shell_get_default ());
-  favicon_db_path = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (mode) ?
-                                      ephy_dot_dir () : g_get_user_cache_dir (),
-                                      g_get_prgname (), "icondatabase", NULL);
-
-  webkit_web_context_set_favicon_database_directory (web_context, favicon_db_path);
-  g_free (favicon_db_path);
 }
 
 static void
@@ -901,42 +874,6 @@ ephy_shell_get_n_windows (EphyShell *shell)
   return g_list_length (list);
 }
 
-EphyWindow*
-ephy_shell_get_main_window (EphyShell *shell)
-{
-  EphyWindow *window = NULL;
-  GList *windows;
-  GList *iter;
-
-  g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
-
-  /* Select the window with most tabs in the current workspace as the window to
-   * use for opening a new tab on, if that turns out to be the case.
-   */
-  windows = gtk_application_get_windows (GTK_APPLICATION (shell));
-
-  for (iter = windows; iter != NULL; iter = iter->next) {
-    EphyWindow *candidate = EPHY_WINDOW (iter->data);
-    GtkNotebook *cur_notebook;
-    GtkNotebook *cand_notebook;
-
-    if (!ephy_window_is_on_current_workspace (candidate))
-      continue;
-
-    if (!window) {
-      window = candidate;
-      continue;
-    }
-
-    cur_notebook = GTK_NOTEBOOK (ephy_window_get_notebook (window));
-    cand_notebook =  GTK_NOTEBOOK (ephy_window_get_notebook (candidate));
-    if (gtk_notebook_get_n_pages (cand_notebook) > gtk_notebook_get_n_pages (cur_notebook))
-      window = candidate;
-  }
-
-  return window;
-}
-
 gboolean
 ephy_shell_close_all_windows (EphyShell *shell)
 {
@@ -962,15 +899,6 @@ ephy_shell_close_all_windows (EphyShell *shell)
   return retval;
 }
 
-static gboolean
-tab_is_empty (EphyEmbed *embed)
-{
-  EphyWebView *view = ephy_embed_get_web_view (embed);
-
-  return ((ephy_web_view_get_is_blank (view) || ephy_web_view_is_overview (view)) &&
-          !ephy_web_view_is_loading (view));
-}
-
 typedef struct {
   EphyShell *shell;
   EphySession *session;
@@ -1012,7 +940,7 @@ open_uris_data_new (EphyShell *shell,
     data->window = ephy_window_new ();
   } else if (startup_flags & EPHY_STARTUP_NEW_TAB || (new_windows_in_tabs && have_uris)) {
     data->flags |= EPHY_NEW_TAB_JUMP;
-    data->window = ephy_shell_get_main_window (shell);
+    data->window = EPHY_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (shell)));
     data->reuse_empty_tab = TRUE;
   } else if (!have_uris) {
     data->window = ephy_window_new ();
@@ -1047,7 +975,9 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
     page_flags |= EPHY_NEW_TAB_APPEND_AFTER;
   else if (data->reuse_empty_tab) {
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (data->window));
-    reusing_empty_tab = tab_is_empty (embed);
+    /* Only load a new page in this embed if it was showing or loading the homepage about:overview */
+    if (ephy_web_view_get_visit_type (ephy_embed_get_web_view (embed)) == EPHY_PAGE_VISIT_HOMEPAGE)
+      reusing_empty_tab = TRUE;
   }
 
   if (!reusing_empty_tab) {
