@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __EPHY_TITLE_BOX_H__
@@ -73,6 +71,9 @@ void                ephy_title_box_set_security_level   (EphyTitleBox         *t
 
 GtkWidget          *ephy_title_box_get_location_entry   (EphyTitleBox         *title_box);
 
+void                ephy_title_box_set_address          (EphyTitleBox         *title_box,
+                                                         const char           *address);
+
 G_END_DECLS
 
 #endif /* __EPHY_TITLE_BOX_H__ */
