@@ -12,9 +12,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
@@ -24,7 +22,6 @@
 #ifndef EPHY_WINDOW_H
 #define EPHY_WINDOW_H
 
-#include "ephy-download.h"
 #include "ephy-web-view.h"
 
 #include <gtk/gtk.h>
@@ -43,8 +40,7 @@ typedef enum
         EPHY_WINDOW_CHROME_TOOLBAR       = 1 << 0,
         EPHY_WINDOW_CHROME_MENU          = 1 << 1,
         EPHY_WINDOW_CHROME_LOCATION      = 1 << 2,
-        EPHY_WINDOW_CHROME_DOWNLOADS_BOX = 1 << 3,
-        EPHY_WINDOW_CHROME_TABSBAR       = 1 << 4,
+        EPHY_WINDOW_CHROME_TABSBAR       = 1 << 3,
         EPHY_WINDOW_CHROME_DEFAULT       = (EPHY_WINDOW_CHROME_TOOLBAR | EPHY_WINDOW_CHROME_MENU | EPHY_WINDOW_CHROME_LOCATION | EPHY_WINDOW_CHROME_TABSBAR)
 } EphyWindowChrome;
 
@@ -84,9 +80,6 @@ const char       *ephy_window_get_location        (EphyWindow *window);
 
 gboolean          ephy_window_close               (EphyWindow *window);
 
-void              ephy_window_add_download        (EphyWindow *window,
-                                                   EphyDownload *download);
-
 EphyWindowChrome  ephy_window_get_chrome          (EphyWindow *window);
 
 gboolean      ephy_window_is_on_current_workspace (EphyWindow *window);
