@@ -13,16 +13,14 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "popup-commands.h"
 
 #include "ephy-bookmarks-ui.h"
-#include "ephy-download.h"
+#include "ephy-downloads-manager.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-chooser.h"
@@ -149,19 +147,21 @@ cancel_download_idle_cb (EphyDownload *download)
 	return FALSE;
 }
 
+typedef struct {
+	char *title;
+	EphyWindow *window;
+} SavePropertyURLData;
+
 static void
 filename_suggested_cb (EphyDownload *download,
 		       const char *suggested_filename,
-		       char *title)
+		       SavePropertyURLData *data)
 {
-	EphyWindow *window;
 	EphyFileChooser *dialog;
 	char *sanitized_filename;
 
-	window = EPHY_WINDOW (ephy_download_get_window (download));
-
-	dialog = ephy_file_chooser_new (title,
-					GTK_WIDGET (window),
+	dialog = ephy_file_chooser_new (data->title,
+					GTK_WIDGET (data->window),
 					GTK_FILE_CHOOSER_ACTION_SAVE,
 					EPHY_FILE_FILTER_NONE);
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
@@ -183,7 +183,8 @@ filename_suggested_cb (EphyDownload *download,
 		webkit_download = ephy_download_get_webkit_download (download);
 		webkit_download_set_allow_overwrite (webkit_download, TRUE);
 
-		ephy_window_add_download (window, download);
+		ephy_downloads_manager_add_download (ephy_embed_shell_get_downloads_manager (ephy_embed_shell_get_default ()),
+						     download);
 	}
 	else
 	{
@@ -194,7 +195,9 @@ filename_suggested_cb (EphyDownload *download,
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
-	g_free (title);
+	g_free (data->title);
+	g_object_unref (data->window);
+	g_free (data);
 
 	g_object_unref (download);
 }
@@ -207,6 +210,7 @@ save_property_url (const char *title,
 	EphyEmbedEvent *event;
 	const char *location;
 	EphyDownload *download;
+	SavePropertyURLData *data;
 	GValue value = { 0, };
 
 	event = ephy_window_get_context_event (window);
@@ -214,10 +218,13 @@ save_property_url (const char *title,
 
 	ephy_embed_event_get_property (event, property, &value);
 	location = g_value_get_string (&value);
-	download = ephy_download_new_for_uri (location, GTK_WINDOW (window));
+	download = ephy_download_new_for_uri (location);
+	data = g_new (SavePropertyURLData, 1);
+	data->title = g_strdup (title);
+	data->window = g_object_ref (window);
 	g_signal_connect (download, "filename-suggested",
 			  G_CALLBACK (filename_suggested_cb),
-			  g_strdup (title));
+			  data);
 
 	g_value_unset (&value);
 }
@@ -271,7 +278,7 @@ popup_cmd_set_image_as_background (GtkAction *action,
 	ephy_embed_event_get_property (event, "image-uri", &value);
 	location = g_value_get_string (&value);
 
-	download = ephy_download_new_for_uri (location, GTK_WINDOW (window));
+	download = ephy_download_new_for_uri (location);
 
 	base = g_path_get_basename (location);
 	base_converted = g_filename_from_utf8 (base, -1, NULL, NULL, NULL);
@@ -280,7 +287,8 @@ popup_cmd_set_image_as_background (GtkAction *action,
 
 	ephy_download_set_destination_uri (download, dest_uri);
 	ephy_download_set_action (download, EPHY_DOWNLOAD_ACTION_DO_NOTHING);
-	ephy_window_add_download (window, download);
+	ephy_downloads_manager_add_download (ephy_embed_shell_get_downloads_manager (ephy_embed_shell_get_default ()),
+					     download);
 	g_object_unref (download);
 
 	g_signal_connect (download, "completed",
@@ -367,3 +375,38 @@ popup_cmd_media_in_new_tab (GtkAction *action,
 {
 	popup_cmd_view_in_new_tab (window, "media-uri");
 }
+
+void
+popup_cmd_link_in_incognito_window (GtkAction *action,
+				    EphyWindow *window)
+{
+	EphyEmbedEvent *event;
+	GValue value = { 0, };
+
+	event = ephy_window_get_context_event (window);
+	g_assert (event != NULL);
+
+	ephy_embed_event_get_property (event, "link-uri", &value);
+	ephy_open_incognito_window (g_value_get_string (&value));
+	g_value_unset (&value);
+}
+
+void
+popup_cmd_search_selection (GtkAction *action,
+			    EphyWindow *window)
+{
+	EphyEmbed *embed, *new_embed;
+	const char *text;
+	char *search_url;
+
+	embed = ephy_embed_container_get_active_child
+		(EPHY_EMBED_CONTAINER (window));
+	g_assert (EPHY_IS_EMBED (embed));
+
+	text = g_object_get_data (G_OBJECT (action), "selection");
+	search_url = ephy_embed_utils_autosearch_address (text);
+	new_embed = ephy_shell_new_tab (ephy_shell_get_default (),
+					window, embed, EPHY_NEW_TAB_APPEND_AFTER);
+	ephy_web_view_load_url (ephy_embed_get_web_view (new_embed), search_url);
+	g_free (search_url);
+}
