@@ -15,9 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -85,8 +83,7 @@ struct PrefsDialogPrivate
 	GtkWidget *never;
 	GtkWidget *remember_passwords_checkbutton;
 	GtkWidget *do_not_track_checkbutton;
-	GtkWidget *disk_cache_spinbutton;
-	GtkWidget *clear_cache_button;
+	GtkWidget *clear_personal_data_button;
 
 	/* language */
 	GtkWidget *default_encoding_combo;
@@ -114,6 +111,7 @@ enum {
 enum
 {
 	SEARCH_ENGINE_COL_NAME,
+	SEARCH_ENGINE_COL_STOCK_URL,
 	SEARCH_ENGINE_COL_URL,
 	SEARCH_ENGINE_NUM_COLS
 };
@@ -145,11 +143,10 @@ static void
 on_manage_cookies_button_clicked (GtkWidget *button,
 				  PrefsDialog *dialog)
 {
-	CookiesDialog *cookies_dialog;
+	EphyCookiesDialog *cookies_dialog;
+
+	cookies_dialog = ephy_cookies_dialog_new ();
 
-	cookies_dialog = g_object_new (EPHY_TYPE_COOKIES_DIALOG,
-				       "use-header-bar", TRUE,
-				       NULL);
 	gtk_window_set_transient_for (GTK_WINDOW (cookies_dialog), GTK_WINDOW (dialog));
 	gtk_window_set_modal (GTK_WINDOW (cookies_dialog), TRUE);
 	gtk_window_present (GTK_WINDOW (cookies_dialog));
@@ -159,11 +156,10 @@ static void
 on_manage_passwords_button_clicked (GtkWidget *button,
 				    PrefsDialog *dialog)
 {
-	PasswordsDialog *passwords_dialog;
+	EphyPasswordsDialog *passwords_dialog;
+
+	passwords_dialog = ephy_passwords_dialog_new ();
 
-	passwords_dialog = g_object_new (EPHY_TYPE_PASSWORDS_DIALOG,
-					 "use-header-bar", TRUE,
-					 NULL);
 	gtk_window_set_transient_for (GTK_WINDOW (passwords_dialog), GTK_WINDOW (dialog));
 	gtk_window_set_modal (GTK_WINDOW (passwords_dialog), TRUE);
 	gtk_window_present (GTK_WINDOW (passwords_dialog));
@@ -203,8 +199,7 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
 	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, never);
 	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, remember_passwords_checkbutton);
 	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, do_not_track_checkbutton);
-	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, disk_cache_spinbutton);
-	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, clear_cache_button);
+	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, clear_personal_data_button);
 
 	/* language */
 	gtk_widget_class_bind_template_child_private (widget_class, PrefsDialog, default_encoding_combo);
@@ -258,6 +253,7 @@ combo_get_mapping (GValue *value,
 		if (g_strcmp0 (item_name, settings_name) == 0)
 		{
 			g_value_set_int (value, i);
+			g_free (item_name);
 			break;
 		}
 
@@ -954,15 +950,14 @@ prefs_dialog_response_cb (GtkDialog *widget,
 }
 
 static void
-clear_cache_button_clicked_cb (GtkWidget *button,
-			       PrefsDialog *dialog)
+clear_personal_data_button_clicked_cb (GtkWidget *button,
+				       PrefsDialog *dialog)
 {
 	ClearDataDialog *clear_dialog;
 
 	clear_dialog = g_object_new (EPHY_TYPE_CLEAR_DATA_DIALOG,
 				     "use-header-bar", TRUE,
 				     NULL);
-	clear_data_dialog_set_flags (clear_dialog, CLEAR_DATA_CACHE);
 	gtk_window_set_transient_for (GTK_WINDOW (clear_dialog), GTK_WINDOW (dialog));
 	gtk_window_set_modal (GTK_WINDOW (clear_dialog), TRUE);
 	gtk_window_present (GTK_WINDOW (clear_dialog));
@@ -1016,6 +1011,7 @@ search_engine_combo_add_default_engines (GtkListStore *store)
 	int i;
 	const char *default_engines[][3] = { /* Search engine option in the preferences dialog */
 					     { N_("DuckDuckGo"),
+					       "https://duckduckgo.com/?q=%s&t=epiphany",
 					       /* For the preferences dialog. Must exactly match the URL
 					        * you chose in the gschema, but with & instead of &amp;
 					        * If the match is not exact, there will be a spurious, ugly
@@ -1023,10 +1019,12 @@ search_engine_combo_add_default_engines (GtkListStore *store)
 					       N_("https://duckduckgo.com/?q=%s&t=epiphany")},
 					     /* Search engine option in the preferences dialog */
 					     { N_("Google"),
+					       "https://google.com/search?q=%s",
 					       /* For the preferences dialog. Consider a regional variant, like google.co.uk */
 					       N_("https://google.com/search?q=%s")},
 					     /* Search engine option in the preferences dialog */
 					     { N_("Bing"),
+					       "https://www.bing.com/search?q=%s",
 					       /* For the preferences dialog. Consider a regional variant, like uk.bing.com */
 					       N_("https://www.bing.com/search?q=%s")} };
 
@@ -1035,8 +1033,10 @@ search_engine_combo_add_default_engines (GtkListStore *store)
 		gtk_list_store_insert_with_values (store, NULL, -1,
 						   SEARCH_ENGINE_COL_NAME,
 						   _(default_engines[i][0]),
+						   SEARCH_ENGINE_COL_STOCK_URL,
+						   default_engines[i][1],
 						   SEARCH_ENGINE_COL_URL,
-						   _(default_engines[i][1]),
+						   _(default_engines[i][2]),
 						   -1);
 	}
 }
@@ -1074,6 +1074,7 @@ search_engine_combo_add_smart_bookmarks (GtkListStore *store)
 
 		gtk_list_store_insert_with_values (store, NULL, -1,
 						   SEARCH_ENGINE_COL_NAME, bookmark_name,
+						   SEARCH_ENGINE_COL_STOCK_URL, bookmark_url,
 						   SEARCH_ENGINE_COL_URL, bookmark_url,
 						   -1);
 	}
@@ -1098,14 +1099,17 @@ search_engine_combo_add_current_engine (GtkListStore *store)
 
 	while (!in_combo && has_next)
 	{
-		char *url;
+		char *stock_url, *url;
 
 		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
+				    SEARCH_ENGINE_COL_STOCK_URL, &stock_url,
 				    SEARCH_ENGINE_COL_URL, &url, -1);
 
-		if (strcmp (original_url, url) == 0)
+		if (strcmp (original_url, stock_url) == 0 ||
+		    strcmp (original_url, url) == 0)
 			in_combo = TRUE;
 
+		g_free (stock_url);
 		g_free (url);
 		has_next = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
 	}
@@ -1113,6 +1117,7 @@ search_engine_combo_add_current_engine (GtkListStore *store)
 	if (!in_combo)
 		gtk_list_store_insert_with_values (store, NULL, -1,
 						   SEARCH_ENGINE_COL_NAME, original_url,
+						   SEARCH_ENGINE_COL_STOCK_URL, original_url,
 						   SEARCH_ENGINE_COL_URL, original_url,
 						   -1);
 
@@ -1283,15 +1288,10 @@ setup_privacy_page (PrefsDialog *dialog)
 			 priv->do_not_track_checkbutton,
 			 "active",
 			 G_SETTINGS_BIND_DEFAULT);
-	g_settings_bind (web_settings,
-			 EPHY_PREFS_CACHE_SIZE,
-			 priv->disk_cache_spinbutton,
-			 "value",
-			 G_SETTINGS_BIND_DEFAULT);
 
-	g_signal_connect (priv->clear_cache_button,
+	g_signal_connect (priv->clear_personal_data_button,
 			  "clicked",
-			  G_CALLBACK (clear_cache_button_clicked_cb),
+			  G_CALLBACK (clear_personal_data_button_clicked_cb),
 			  dialog);
 }
 
