@@ -12,9 +12,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
@@ -84,6 +82,8 @@ void window_cmd_view_zoom_normal          (GtkAction  *action,
                                            EphyWindow *window);
 void window_cmd_view_page_source          (GtkAction  *action,
                                            EphyWindow *window);
+void window_cmd_help_contents             (GtkAction  *action,
+                                           GtkWidget  *window);
 void window_cmd_help_about                (GtkAction  *action,
                                            GtkWidget  *window);
 void window_cmd_tabs_next                 (GtkAction  *action,
@@ -112,8 +112,6 @@ void window_cmd_file_new_incognito_window (GtkAction  *action,
                                            EphyWindow *window);
 void window_cmd_edit_preferences          (GtkAction  *action,
                                            EphyWindow *window);
-void window_cmd_edit_personal_data        (GtkAction  *action,
-                                           EphyWindow *window);
 
 G_END_DECLS
 
