@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Epiphany; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA  02110-1301  USA
+ * along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -23,7 +21,9 @@
 
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
+#include "ephy-private.h"
 #include "ephy-profile-utils.h"
+#include "ephy-shell.h"
 
 const char* bookmarks_paths[] = { EPHY_BOOKMARKS_FILE, EPHY_BOOKMARKS_FILE_RDF };
 
@@ -112,6 +112,8 @@ main (int argc, char *argv[])
     return -1;
   }
 
+  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_TEST);
+
   g_test_add_func ("/src/bookmarks/ephy-bookmarks/create",
                    test_ephy_bookmarks_create);
 
