@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Epiphany; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA  02110-1301  USA
+ * along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -32,7 +30,7 @@ test_ephy_completion_model_create (void)
 {
     EphyCompletionModel *model;
     model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ())),
-                                       ephy_shell_get_bookmarks (ephy_shell_get_default ()));
+                                       ephy_shell_get_bookmarks (ephy_shell_get_default ()), TRUE);
     g_assert (model);
     g_object_unref (model);
 }
@@ -58,7 +56,7 @@ test_ephy_completion_model_update_empty (void)
     GMainLoop *loop = NULL;
 
     model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ())),
-                                       ephy_shell_get_bookmarks (ephy_shell_get_default ()));
+                                       ephy_shell_get_bookmarks (ephy_shell_get_default ()), TRUE);
     g_assert (model);
 
     loop = g_main_loop_new (NULL, FALSE);
