@@ -17,9 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Epiphany; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA  02110-1301  USA
+ * along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -89,17 +87,12 @@ test_ephy_file_helpers_init (void)
 
     /* Cleanup dir left behind. */
     if (keep_dir) {
-      GFile *file;
-
-      file = g_file_new_for_path (tmp_dir);
       /* As a safety measure, only try recursive delete on paths
        * prefixed with /tmp. */
       if (g_str_has_prefix (tmp_dir, "/tmp"))
-        g_assert (ephy_file_delete_dir_recursively (file, NULL));
+        g_assert (ephy_file_delete_dir_recursively (tmp_dir, NULL));
       else
         g_warning ("INIT: dangerous path returned as tmp_dir: %s", tmp_dir);
-
-      g_object_unref (file);
     }
 
     g_free (tmp_dir);
@@ -184,8 +177,8 @@ test_ephy_file_create_delete_dir (void)
   ephy_file_helpers_init (NULL, EPHY_FILE_HELPERS_PRIVATE_PROFILE, NULL);
 
   for (i = 0; i < G_N_ELEMENTS (dir_tests); i++) {
-    GFile *file = NULL;
     DirTest test;
+    GError *error = NULL;
 
     test = dir_tests[i];
 
@@ -195,14 +188,12 @@ test_ephy_file_create_delete_dir (void)
     g_assert (ephy_ensure_dir_exists (test.dir, NULL) == (test.exists || test.can_create));
     g_assert (g_file_test (test.dir, G_FILE_TEST_EXISTS) == (test.exists || test.can_create));
 
-    file = g_file_new_for_path (test.dir);
-
-    g_assert (ephy_file_delete_dir_recursively (file, NULL) == test.can_delete);
+    g_assert (ephy_file_delete_dir_recursively (test.dir, &error) == test.can_delete);
+    if (error)
+            g_error_free (error);
 
     if (test.exists)
       g_assert (g_file_test (test.dir, G_FILE_TEST_EXISTS) != test.can_delete);
-
-    g_object_unref (file);
   }
 
   ephy_file_helpers_shutdown ();
@@ -290,76 +281,6 @@ test_ephy_file_create_delete_tmp (void)
   ephy_file_helpers_shutdown ();
 }
 
-static void
-test_ephy_file_switch_temp_file (void)
-{
-  char *tmp_file;
-
-  GFile *orig;
-  char *orig_path;
-
-  GFile *dest;
-  char *dest_path;
-  char *file_cont = NULL;
-
-  ephy_file_helpers_init (NULL, EPHY_FILE_HELPERS_PRIVATE_PROFILE, NULL);
-
-  /* Empty dest */
-  tmp_file = ephy_file_tmp_filename ("test-dest-XXXXXX", NULL);
-  dest_path = g_build_filename (ephy_file_tmp_dir (), tmp_file, NULL);
-  g_free (tmp_file);
-
-  g_assert (g_file_test (dest_path, G_FILE_TEST_EXISTS) == FALSE);
-  dest = g_file_new_for_path (dest_path);
-
-  tmp_file = ephy_file_tmp_filename ("test-orig-XXXXXX", NULL);
-  orig_path = g_build_filename (ephy_file_tmp_dir (), tmp_file, NULL);
-  g_free (tmp_file);
-
-  g_assert (g_file_test (orig_path, G_FILE_TEST_EXISTS) == FALSE);
-  orig = g_file_new_for_path (orig_path);
-
-  g_file_set_contents (orig_path, "orig", -1, NULL);
-  g_assert (g_file_test (orig_path, G_FILE_TEST_EXISTS));
-
-  g_test_message ("SWITCH: %s to %s", orig_path, dest_path);
-
-  g_assert (ephy_file_switch_temp_file (dest, orig));
-  g_assert (g_file_test (orig_path, G_FILE_TEST_EXISTS) == FALSE);
-  g_assert (g_file_test (dest_path, G_FILE_TEST_EXISTS));
-
-  g_assert (g_file_get_contents (dest_path, &file_cont, NULL, NULL));
-  g_assert_cmpstr ("orig", ==, file_cont);
-  g_free (file_cont);
-
-  ephy_file_delete_uri (g_file_get_uri (dest));
-  g_assert (g_file_test (dest_path, G_FILE_TEST_EXISTS) == FALSE);
-
-  /* Full replace */
-  g_file_set_contents (dest_path, "dest", -1, NULL);
-  g_assert (g_file_test (dest_path, G_FILE_TEST_EXISTS));
-
-  g_file_set_contents (orig_path, "orig", -1, NULL);
-  g_assert (g_file_test (orig_path, G_FILE_TEST_EXISTS));
-
-  g_test_message ("SWITCH REPLACE: %s to %s", orig_path, dest_path);
-  g_assert (ephy_file_switch_temp_file (dest, orig));
-  g_assert (g_file_test (dest_path, G_FILE_TEST_EXISTS));
-  g_assert (g_file_test (orig_path, G_FILE_TEST_EXISTS) == FALSE);
-
-  g_assert (g_file_get_contents (dest_path, &file_cont, NULL, NULL));
-  g_assert_cmpstr ("orig", ==, file_cont);
-  g_free (file_cont);
-
-  g_free (orig_path);
-  g_free (dest_path);
-
-  g_object_unref (orig);
-  g_object_unref (dest);
-
-  ephy_file_helpers_shutdown ();
-}
-
 typedef struct {
   const char *filename;
   const char *expected;
@@ -425,9 +346,6 @@ main (int argc, char *argv[])
   g_test_add_func ("/lib/ephy-file-helpers/create_delete_tmp",
                    test_ephy_file_create_delete_tmp);
 
-  g_test_add_func ("/lib/ephy-file-helpers/switch_temp_file",
-                   test_ephy_file_switch_temp_file);
-
   g_test_add_func ("/lib/ephy-file-helpers/sanitize_filename",
                    test_ephy_sanitize_filename);
 
